-------------------------------------------------------------------------
--
-- File name    :  Microsequencer
-- Title        :  Microsequencer
-- Library      :  WORK
--              :  
-- Purpose      :  
--              : 
-- Created On   : 08/24/06 12:51:11
--              :
-- Comments     : 
--              : 
-- Assumptions  : start register
-- Limitations  : none
-- Developers   : Juan de Vicente
--              : 
-- Notes        :
--
-- PATTERN MEMORY: 
-- Port A:  2x256x16 Address (16 data bus)  = 0x1000 => 0x11FF for loading microsequencer data 
-- Port B:  256x32   For running the Microsequencer
--
-- 
-- Load by port A -> Consecutive positions in different modules  (low interlaced)
--
--**************************************************************
--                      CB microsequencer pattern memory
--**************************************************************
--          D31      D30 ...........D12 D11........D8 D7..........D0
--    Not_used   Horizontal_clocks       Not_used         Delay
--**************************************************************
--
-- **************************************************************
--         Microsequencer Trigger Command Data Word Format
-- **************************************************************
--    D15         D14..................D8  D7....................D0                          
--    Not used    run_length of sequence   start address of sequence
-- **************************************************************
-- ----------------------------------------------------------------------
-- Revision History :
-- ----------------------------------------------------------------------
--   Version No:| Juan de Vicente   | Changes Made:                | Mod. Date:
--    v1.0     | Juan de Vicente    | Automatically Generated      | 08/24/06    
--    v1.1     | Juan de Vicente    | 2 RAM memory modules         | 11/30/2006
--             |                    | (low Intelaced in port A)    | 
--    v1.2     | Jamieson Olsen     | new FSM, lowered latency     | 12 Aug 2009
--             |                    | removed wait state           |
--             |                    | register async inputs        |
-- ----------------------------------------------------------------------
--
--DESCRIPTION: This module generates an horizontal clock sequence previousely loaded

--Port definitions:

--addr_backplane:        address settled from the backplane interface        
--data_in_backplane:     data coming from the backplane interface  
--data_out_backplane:    data bus used for reading the pattern memory     
--sequencer_trigger_out: data bus used for reading the microsequencer trigger command register
--clk:                   system clock                     
--cs:                    chip select for pattern memory                    
--cs_micro_control_reg:  enable microsequencer trigger command register
--ena_micro:             multiplexor control signal to select between microsequencer and sequencer  
--rst:                   reset from backplane, active high
--wr_backplane:          16 bit write            
--clocks:                clock sequence output

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

library unisim;
use unisim.vcomponents.all;

entity Microsequencer is
  port (ADDR_BACKPLANE        : in  std_logic_vector (8 downto 0);
        DATA_IN_BACKPLANE     : in  std_logic_vector (15 downto 0);
        DATA_OUT_BACKPLANE    : out std_logic_vector (15 downto 0);
        SEQUENCER_TRIGGER_OUT : out std_logic_vector(15 downto 0);
        CLK                   : in  std_logic;
        CS                    : in  std_logic;
        CS_MICRO_CONTROL_REG  : in  std_logic;
        ENA_MICRO             : out std_logic;
        RST                   : in  std_logic;
        WR_BACKPLANE          : in  std_logic;
        WR32                  : in  std_logic;
        CLOCKS                : out std_logic_vector (30 downto 12)); 
end Microsequencer;

architecture Microsequencer_arch of Microsequencer is

  signal DOA           : std_logic_vector(31 downto 0);
  signal DOB           : std_logic_vector(31 downto 0);
  signal cs0,cs1       : std_logic;
  signal invclk        : std_logic;

  signal addr_reg, start_reg, end_reg, delay_reg : std_logic_vector(7 downto 0);
  type   state_type is (reset, idle, run, delay);
  signal state                                   : state_type;
  signal clocks_reg, clocks1_reg                 : std_logic_vector(30 downto 12);
  signal ena_micro_reg                           : std_logic_vector(1 downto 0);

  signal go_reg : std_logic; 
  signal data_in_backplane_reg: std_logic_vector (15 downto 0);

begin

  -- invert the clock to be used only the port B side of 
  -- the block rams.  this inverter will get absorbed into the 
  -- block ram itself, so no gated clocks will be created.

  invclk <= not clk;

  cs0 <= cs and (not ADDR_BACKPLANE(0));
  cs1 <= cs and ADDR_BACKPLANE(0);

  -- Register the inputs to this module because some of them are 
  -- async and can cause the FSM to get into a bad state.

  inproc: process(CLK)
  begin
    if rising_edge(clk) then
      if (RST = '1') then
        go_reg <= '0';
        data_in_backplane_reg <= (others=>'0');
      else
        if (WR32 = '1' and CS_MICRO_CONTROL_REG = '1' and (data_in_backplane(15 downto 8) /= X"00")) then
            go_reg <= '1';
        else
            go_reg <= '0';
        end if;
        data_in_backplane_reg <= data_in_backplane;
      end if;
    end if;
  end process inproc;


  RAMB4_S16_S16_inst0 : RAMB4_S16_S16
    generic map (
      INIT_00 => X"F0200020F0200020F0200020F0200020F0200020F0200020F0200020F0200020",  --relojes horizontales de 1.280 us
      INIT_01 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_02 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_03 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
    port map (
      CLKA  => clk,                     -- Port A clock input
      ADDRA => addr_backplane(8 downto 1),  -- Port A 8-bit address input
      DIA   => data_in_backplane,       -- Port A 16-bit data input
      ENA   => cs0,                     -- Port A RAM enable input
      RSTA  => rst,                     -- Port A Synchronous reset input
      WEA   => wr_backplane,            -- Port A RAM write enable input
      DOA   => DOA(15 downto 0),        -- Port A 16-bit data output
      ------- Port A = backplane, port B = sequencer      
      CLKB  => invclk,                  -- Port B clock input
      ADDRB => addr_reg,                -- Port B 8-bit address input
      DIB   => X"0000",                 -- Port B 16-bit data input
      ENB   => '1',                     -- Port B RAM enable input
      RSTB  => rst,                     -- Port B Synchronous reset input
      WEB   => '0',                     -- Port B RAM write enable input
      DOB   => DOB(15 downto 0)         -- Port B 16-bit data output
      );

  RAMB4_S16_S16_inst1 : RAMB4_S16_S16
    generic map (
      INIT_00 => X"FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000",
      INIT_01 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_02 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_03 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
    port map (
      CLKA  => clk,                         -- Port A clock input
      ADDRA => addr_backplane(8 downto 1),  -- Port A 8-bit address input
      DIA   => data_in_backplane,           -- Port A 16-bit data input
      ENA   => cs1,                         -- Port A RAM enable input
      RSTA  => rst,                         -- Port A Synchronous reset input
      WEA   => wr_backplane,                -- Port A RAM write enable input
      DOA   => DOA(31 downto 16),           -- Port A 16-bit data output
      ------- Port A = backplane, port B = sequencer
      CLKB  => invclk,                      -- Port B clock input
      ADDRB => addr_reg,                    -- Port B 8-bit address input
      DIB   => X"0000",                     -- Port B 16-bit data input
      ENB   => '1',                         -- Port B RAM enable input
      RSTB  => rst,                         -- Port B Synchronous reset input
      WEB   => '0',                         -- Port B RAM write enable input
      DOB   => DOB(31 downto 16)            -- Port B 16-bit data output
      );

  -- the goal is to make this sequencer look like the seq in the ACQ12 board
  -- where state with a delay value of zero = 25 and there are no extra clock
  -- cycles used to transition to the next state.

  -- in order to make this work the FSM increments the address, then needs
  -- to know the delay value *before* the next clock cycle to know if it should
  -- increment the address pointer to hold for a delay.  The only way to make
  -- this work with a block ram is to clock the block ram on the falling edge 
  -- of the clock.

  fsm : process(CLK)
  begin
    if rising_edge(clk) then
      if (RST = '1') then
        start_reg     <= (others => '0');
        end_reg       <= (others => '0');
        addr_reg      <= (others => '0');
        delay_reg     <= (others => '0');
        state         <= reset;
        ena_micro_reg <= "00";
        clocks_reg    <= (others => '0');
        clocks1_reg    <= (others => '0');

      else
        case state is

          when reset =>
            state <= idle;

          when idle =>                  -- don't start if run length = 0 
            if (go_reg='1') then
              start_reg     <= data_in_backplane_reg(7 downto 0);  -- starting address
              addr_reg      <= data_in_backplane_reg(7 downto 0);  -- address pointer
              end_reg       <= (data_in_backplane_reg(7 downto 0) + data_in_backplane_reg(15 downto 8) - 1);  -- start_address + run_length
              state         <= run;
            else
              ena_micro_reg(0) <= '0';
              clocks_reg    <= (others => '0');
            end if;

          when run =>
            clocks_reg    <= dob(30 downto 12);
            ena_micro_reg(0) <= '1';
            if (addr_reg = end_reg) then        -- last state 
              if (DOB(7 downto 0) = "00") then  -- zero delay, done
                state      <= idle;
              else                      -- finish last delay cycle, then idle.
                state     <= delay;
                delay_reg <= DOB(7 downto 0);
              end if;
            else
              if (DOB(7 downto 0) = "00") then  -- zero delay
                state    <= run;
                addr_reg <= addr_reg + 1;
              else
                state     <= delay;
                delay_reg <= DOB(7 downto 0);
              end if;
            end if;
            
          when delay =>
            if (delay_reg = X"01") then
              if (addr_reg = end_reg) then
                state      <= idle;
              else
                state    <= run;
                addr_reg <= addr_reg + 1;
              end if;
            else
              delay_reg <= delay_reg - 1;
              state     <= delay;
            end if;

          when others =>
            state <= reset;
            
        end case;

        -- add delay to line up with ACQ12 sequencer

        clocks1_reg <= clocks_reg;
        ena_micro_reg(1) <= ena_micro_reg(0);

      end if;
    end if;
  end process;

  DATA_OUT_BACKPLANE <= DOA(15 downto 0) when cs0 = '1' else
                        DOA(31 downto 16) when cs1 = '1' else
                        (others => 'Z');
  
  CLOCKS(30 downto 12) <= clocks1_reg;
  ENA_MICRO <= ena_micro_reg(1);

  SEQUENCER_TRIGGER_OUT <= addr_reg & start_reg;

end Microsequencer_arch;
