------------------------------------------------------------------------
-- PROJECT      :   DES (ADAPTED FORM MONSOON)
-- BOARD        :   CCD CLOCK BOARD
-- DEVICE ID    :   SPARTAN FPGA (U116)
-- PARENT MODULE:   TOP LEVEL (CB_FPGA)  
-- MODULE       :   PERSONALITY MODULE
-- VERSION      :   4.0
------------------------------------------------------------------------
-- FILE NAME    :   PERSONALITYMODULE_v40.VHD
-- AUTHOR       :   JUAN DE VICENTE (Original -  PETER MOORE (NOAO))
-- DATE         :   May/03/2004
-- COMPANY      :   CIEMAT
-- PLATFORM     :   Xilinx ISE Webpack 5.1
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- THIS MODULE PROVIDES THE LOGIC MECHANISM TO INTERFACE THE SEQUENCER
-- BUS INTERFACE TO ANY PARTICULAR FPGA DESIGN. 
-- IT PROVIDES FOR LOCAL ADDRESS DECODE LOGIC AND A SIMPLE DATA SELECTION
-- MUX TO PERSONALIZE YOUR DESIGN TO A PARTICULAR FPGA LOGIC IMPLEMENTATION 
-- TO THE SEQUENCER INTERFACE LOGIC BLOCK.
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
--
-- Memory Map for this board is:
--
-- FIXED ADDRESS ASSIGNMENTS
--
-- LED INDICATOR SELECT PORT     = 0xFFF9 (3 BITS WRITE - 3 BITS READ)
-- CIRCUIT BOARD SERIAL NUMBER   = 0xFFFA (NO WRITE - 48 BITS READ)
-- CIRCUIT BOARD TEMPERATURE     = 0xFFFB (NO WRITE - 16 BITS READ)
-- CONTROL REGISTER ADDRESS      = 0xFFFC (16 BIT WRITE - 16 BIT READ)
-- STATUS REGISTER ADDRESS       = 0XFFFD (NO WRITE - 16 BIT READ) 
-- BOARD ID REGISTER ADDRESS     = 0xFFFE (NO WRITE - 16 BIT READ)
-- CODE ID REGISTER ADDRESS      = 0xFFFF (NO WRITE - 16 BIT READ)
--
-- CCD DES CLOCK BOARD SPECIFIC ADDRESS MAP
--
-- CLOCKS REGISTER               = 0x0000 (32 BIT WRITE - 32 BIT READ)
-- GUIDER REGISTERS              = 0x0001 => 0x0004 (32 BIT WRITE - 32 BIT READ)
-- MICROSEQUENCER CONTROL REG (CLK_SEQTRG)   = 0x0005 (32 BIT WRITE - 32 BIT READ)
-- UNUSED                        = 0x0006 => 0x00FF
-- DAC CONFIG 				     = 0x0100 => 0x0163 (8 BIT WRITE - 8 BIT READ)
-- UNUSED                        = 0X0164 => 0X016F
-- ADC CONFIG 					 = 0x0170 => 0x01F7 (8 BIT WRITE - 8 BIT READ)
-- UNUSED                        = 0x01F8 => 0x01FD
-- OUTPUT ENABLES                = 0x01FE (4 BIT WRITE - 4 BIT READ)
-- CLK MUX SELECTIONS            = 0x01FF (16 BIT WRITE - 16 BIT READ) 
-- UNUSED                        = 0x0200 => 0x026F
-- ADC DATA 				     = 0x0270 => 0x02F7 (WRITE TRIGGERS READOUT - 16 BIT READ)

-- MICROSEQUENCER PATTERN MEMORY = 0x1000 => 0x11FF	 (16 BIT WRITE-16 BIT READ)

-- UNUSED                        = 0x02F8 => 0xFFF8
--
-- The SSN is read right after power-up, and is available in the corresponding
-- register after app 7ms.
--
-- The STATUS word is formed by the 8-bit status from the CFG module (7-0) and the
-- CRC_OK bit (in position 8)
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE	      AUTHOR  VERSION  MODIFICATIONS
-- 02/04/2003  PCM      0.1    ORIGINAL VERSION
-- Mar/03       GR      1.1    Personalized for C&B board.
-- 20/May/03    GR      1.2    Added registration of 32-bit data addresses
--                             to allow readback of clock/fbias data.
-- 16/Apr/04    GR      1.3    Added Silicon Serial Number (SSN) and Board Temp registers.
--                             Replaced STATUS_IN(15:0) port by CFG_STATUS_IN(7:0).
--                             Added ports SSN_CRC_OK, SSN_DATA_IN, TEMP_DATA_IN and GET_TEMP_CMD.
-- 03/May/04    GR      1.4    Normalized to new sequencer interface.
-- 20/June/04   GR      4.0    Added new registers for hardware revA
-- 19/June/06   JU      Des    Added microsequencer pattern memory and microsequencer trigger register
-- 18/Jun/10	 JU		Des	 Added guider addresses	 (Nick Langellier)
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- WRITE         : Write enable for write strobes.
-- BOARD_SEL     : Global enable. Signifies that the board has been selected.  
-- ADDRESS       : Address bus input for local decoding.
-- SEQ_DATA_IN   : Sequencer data bus input.
-- DATA_OUT      : 48 bit data bus connected to the pixel data bus through the seq_ifc.
--
-- THE FOLLOWING PORTS WERE ADDED TO TEMPLATE
-- TEL_DATA_IN   : 16 BIT INPUT DATA FROM TEL DATA REGISTER FILE
-- CFG_DATA_IN   : 16 BIT INPUT DATA FROM CFG DATA REGISTER FILE
-- STATUS_IN     : 16 BIT STATUS REGISTER INPUT
-- CS_CFG_REG    : CHIP SELECT FOR CFG DATA REGISTER FILE AT ADDRESSES 0100 => 01FF
-- CS_TEL_REG    : CHIP SELECT FOR TEL DATA REGISTER FILE AT ADDRESSES 0200 => 02FF
-- SSN_CRC_OK    : CRC OK FOR THE SILICON SERIAL NUMBER (SSN)
-- SSN_DATA_IN   : 48 BIT INPUT DATA FROM THE SSN
-- TEMP_DATA_IN  : 12 BIT DATA FOR THE BOARD TEMPERATURE SENSOR
-- GET_TEMP_CMD  : STROBE SIGNAL TO TRIGGER THE BOARD TEMPERATURE READOUT
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity cb_personality_V40 is
    Port ( SYSCLK         				   : in std_logic;
           WRITE          				   : in std_logic;
           BOARD_SEL      				   : in std_logic;
           ADDRESS                		   : in std_logic_vector(15 downto 0);  
           SEQ_DATA_IN    				   : in std_logic_vector(31 downto 0);
           DATA_OUT       				   : out std_logic_vector(47 downto 0);
           TEL_DATA_IN    				   : in std_logic_vector (15 downto 0);
           CFG_DATA_IN    				   : in std_logic_vector (15 downto 0);
		   MICROSEQUENCER_DATA_IN		   : in std_logic_vector (15 downto 0);
		   MICRO_CONTROL_IN				   : in std_logic_vector(15 downto 0);
		   CFG_STATUS_IN  				   : in std_logic_vector(7 downto 0);
           CS_CFG_REG     				   : out std_logic;
           CS_TEL_REG     				   : out std_logic;
           SSN_CRC_OK     				   : in std_logic;
           SSN_DATA_IN    				   : in std_logic_vector(31 downto 0);
           TEMP_DATA_IN   				   : in std_logic_vector(9 downto 0);
           CS_TEMP	    				   : out std_logic;
           LED_DATA_IN    				   : in  std_logic_vector(2 downto 0);
           LED_REG_CS     				   : out std_logic;
		   CS_MUX		   			   	   : out std_logic;
		   CS_OE		  				   : out std_logic;
		   CS_PAT_MEM	  				   : out std_logic;
		   CS_MICRO_CONTROL  		       : out std_logic;
				ANALOG_VOLTAGE_OK					:in std_logic;
			GUIDER_REG                    : in  std_logic_vector(47 downto 0)
			);		   --CLK_SEQTRG_EN
		   
end cb_personality_V40;
	
architecture Behavioral of cb_personality_V40 is

    component Identity_V14 is
    Port ( board_ident_out  : out std_logic_vector(15 downto 0);
           code_ident_out   : out std_logic_vector(15 downto 0)
         );
    end component;

    signal code_ident, board_ident : std_logic_vector(15 downto 0);

    component tristate_output_buffr is
    Port ( DOUT   : out std_logic_vector(63 downto 0);
           DIN    : in std_logic_vector(63 downto 0);
           ENBL_1 : in std_logic;
           ENBL_2 : in std_logic;
           ENBL_3 : in std_logic;
           ENBL_4 : in std_logic);
    end component;

    signal data_in : std_logic_vector(63 downto 0);
    signal control_reg : std_logic_vector(15 downto 0);
    signal status_reg : std_logic_vector(15 downto 0);

-- Decoding Vectors. These are standard decode signals.
    signal CS16K_0, CS16K_1, CS16K_2, CS16K_3 : std_logic;
    signal CS4K_0, CS4K_1, CS4K_2, CS4K_3 : std_logic;
    signal CS1K_0, CS1K_1, CS1K_2, CS1K_3 : std_logic;
    signal CS512_0, CS512_1:std_logic; 
    signal CS256_0, CS256_1, CS256_2, CS256_3 : std_logic;
    signal CS64_0, CS64_1, CS64_2, CS64_3 : std_logic;
    signal CS16_0, CS16_1, CS16_2, CS16_3 : std_logic;
    signal CS_0, CS_1, CS_2, CS_3, CS_4, CS_5, CS_6, CS_7 : std_logic;
    signal CS_8, CS_9, CS_A, CS_B, CS_C, CS_D, CS_E, CS_F : std_logic;
     
-- Standard address select signals     
    signal control_cs, status_cs, board_cs, code_cs : std_logic; -- std chip selects for board regs

-- C&B board specific address select signals
    signal cs_cfg_fb : std_logic;    -- decoded chip select for fast bias cfg data
    signal cs_tel, cs_cfg : std_logic;    -- decoded chip selects for adc data and cfg data
    signal enbl_16, enbl_32 : std_logic;  -- used for selective tristate enables
    signal boardtemp_cs, SNumber_cs: std_logic;  -- register selects for board temp and ssn
    signal LedReg_cs : std_logic;  -- register select for LED data

-- 32-bit data register
    signal seq_clock_reg : std_logic_vector(31 downto 0);

    signal cs_seq_clock : std_logic;
    	
	signal cs_pat_mem_int :std_logic;
	signal cs_micro_control_int :std_logic;
	signal cs_mux_int:std_logic;

-- Skip registers at 0x0001, 0x0002, 0x0003, and 0x0004
	signal cs_ccd1 : std_logic;
	signal cs_ccd2 : std_logic;
	signal cs_ccd3 : std_logic;
	signal cs_load : std_logic;

   begin

------------------------------------------------------------------------
-- BOARD ADDRESS DECODE LOGIC. Enable the decode strobes as required.
------------------------------------------------------------------------

-- Decode 16K word Blocks
    CS16K_0 <= (not ADDRESS(15)) and (not ADDRESS(14));   -- 0x0000 => 0x3FFF
--    CS16K_1 <= (not ADDRESS(15)) and ADDRESS(14);         -- 0x4000 => 0x7FFF
--    CS16K_2 <= ADDRESS(15) and (not ADDRESS(14));         -- 0x8000 => 0xBFFF
    CS16K_3 <= ADDRESS(15) and ADDRESS(14);               -- 0xC000 => 0xFFFF

-- Decode 4K word Blocks
    CS4K_0 <= (not ADDRESS(13)) and (not ADDRESS(12));    --0x0000 => 0x0FFF, 0x4000 => 0x4FFF, 0x8000 => 0x8FFF, 0xC000 => 0xCFFF
    CS4K_1 <= (not ADDRESS(13)) and ADDRESS(12);          --0x1000 => 0x1FFF, 0x5000 => 0x5FFF, 0x9000 => 0x9FFF, 0xD000 => 0xDFFF
--    CS4K_2 <= ADDRESS(13) and (not ADDRESS(12));          --0x2000 => 0x2FFF, 0x6000 => 0x6FFF, 0xA000 => 0xAFFF, 0xE000 => 0xEFFF
    CS4K_3 <= ADDRESS(13) and ADDRESS(12);                -- 0x3000 => 0x3FFF, 0x7000 => 0x7FFF, 0xB000 => 0xBFFF, 0xF000 => 0xFFFF
     
-- Decode 1K word Blocks
    CS1K_0 <= (not ADDRESS(11)) and (not ADDRESS(10));   -- 0x_000 => 0x_3FF
--    CS1K_1 <= (not ADDRESS(11)) and ADDRESS(10);         -- 0x_400 => 0x_7FF
--    CS1K_2 <= ADDRESS(11) and (not ADDRESS(10));         -- 0x_800 => 0x_BFF
    CS1K_3 <= ADDRESS(11) and ADDRESS(10);               -- 0x_C00 => 0x_FFF
-- Decode 512 word Blocks
  	CS512_0 <= not ADDRESS(9);
  	CS512_1 <= ADDRESS(9);
-- Decode 256 word Blocks
    CS256_0 <= (not ADDRESS(9)) and (not ADDRESS(8));    --0x_000 => 0x_0FF, 0x_400 => 0x_4FF, 0x_800 => 0x_8FF, 0x_C00 => 0x_CFF
    CS256_1 <= (not ADDRESS(9)) and ADDRESS(8);          --0x_100 => 0x_1FF, 0x_500 => 0x_5FF, 0x_900 => 0x_9FF, 0x_D00 => 0x_DFF
    CS256_2 <= ADDRESS(9) and (not ADDRESS(8));          --0x_200 => 0x_2FF, 0x_600 => 0x_6FF, 0x_A00 => 0x_AFF, 0x_E00 => 0x_EFF
    CS256_3 <= ADDRESS(9) and ADDRESS(8);                -- 0x_300 => 0x_3FF, 0x_700 => 0x_7FF, 0x_B00 => 0x_BFF, 0x_F00 => 0x_FFF

-- Decode 64 word Blocks
    CS64_0 <= (not ADDRESS(7)) and (not ADDRESS(6));   -- 0x__00 => 0x__3F
 --  CS64_1 <= (not ADDRESS(7)) and ADDRESS(6);         -- 0x__40 => 0x__7F
--    CS64_2 <= ADDRESS(7) and (not ADDRESS(6));         -- 0x__80 => 0x__BF
    CS64_3 <= ADDRESS(7) and ADDRESS(6);               -- 0x__C0 => 0x__FF

 -- Decode 16 word Blocks
    CS16_0 <= (not ADDRESS(5)) and (not ADDRESS(4));    --0x__00 => 0x__0F, 0x__40 => 0x__4F, 0x__80 => 0x__8F, 0x__C0 => 0x__CF
    CS16_1 <= (not ADDRESS(5)) and ADDRESS(4);          --0x__10 => 0x__1F, 0x__50 => 0x__5F, 0x__90 => 0x__9F, 0x__D0 => 0x__DF
    CS16_2 <= ADDRESS(5) and (not ADDRESS(4));          --0x__20 => 0x__2F, 0x__60 => 0x__6F, 0x__A0 => 0x__AF, 0x__E0 => 0x__EF
    CS16_3 <= ADDRESS(5) and ADDRESS(4);                -- 0x__30 => 0x__3F, 0x__70 => 0x__7F, 0x__B0 => 0x__BF, 0x__F0 => 0x__FF

-- Decode single word address lines
    CS_0 <= (not ADDRESS(3)) and (not ADDRESS(2)) and (not ADDRESS(1)) and (not ADDRESS(0));
    CS_1 <= (not ADDRESS(3)) and (not ADDRESS(2)) and (not ADDRESS(1)) and ADDRESS(0);
    CS_2 <= (not ADDRESS(3)) and (not ADDRESS(2)) and ADDRESS(1) and (not ADDRESS(0));
    CS_3 <= (not ADDRESS(3)) and (not ADDRESS(2)) and ADDRESS(1) and ADDRESS(0);
	 CS_4 <= (not ADDRESS(3)) and ADDRESS(2) and (not ADDRESS(1)) and (not ADDRESS(0));
	  CS_5 <= (not ADDRESS(3)) and ADDRESS(2) and (not ADDRESS(1)) and ADDRESS(0);
--    CS_6 <= (not ADDRESS(3)) and ADDRESS(2) and ADDRESS(1) and (not ADDRESS(0));
--    CS_7 <= (not ADDRESS(3)) and ADDRESS(2) and ADDRESS(1) and ADDRESS(0);
--    CS_8 <= ADDRESS(3) and (not ADDRESS(2)) and (not ADDRESS(1)) and (not ADDRESS(0));
    CS_9 <= ADDRESS(3) and (not ADDRESS(2)) and (not ADDRESS(1)) and ADDRESS(0);
    CS_A <= ADDRESS(3) and (not ADDRESS(2)) and ADDRESS(1) and (not ADDRESS(0));
    CS_B <= ADDRESS(3) and (not ADDRESS(2)) and ADDRESS(1) and ADDRESS(0);
    CS_C <= ADDRESS(3) and ADDRESS(2) and (not ADDRESS(1)) and (not ADDRESS(0));
    CS_D <= ADDRESS(3) and ADDRESS(2) and (not ADDRESS(1)) and ADDRESS(0);
    CS_E <= ADDRESS(3) and ADDRESS(2) and ADDRESS(1) and (not ADDRESS(0));
    CS_F <= ADDRESS(3) and ADDRESS(2) and ADDRESS(1) and ADDRESS(0);


-- Specific decode values for interface registers

-- LED indicator register at address 0xfff9 - read / write
    LedReg_cs <= BOARD_SEL and CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_9;
    LED_REG_CS <= LedReg_cs;

-- Circuit board serial number at address 0xfffa - read only
    SNumber_cs <= BOARD_SEL and CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_A;

-- Cicuit board temperature at address 0xfffb - read only
    boardtemp_cs <= BOARD_SEL and CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_B;
	CS_TEMP <= boardtemp_cs;

-- Control Register select at 0xfffc - read / write
    control_cs <= BOARD_SEL and CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_C;

-- Status Register select at 0xfffd - read only
    status_cs <= BOARD_SEL and CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_D;

-- Board id Register select at 0xfffe - read circuit board product code / write is soft reset
    board_cs <= BOARD_SEL and CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_E;

-- Code id Register select at 0xffff - read circuit board firmware idntity / write hard reset (reboot board).
    code_cs <= BOARD_SEL and CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_F;

-- Specific decode values for IR Clock&Bias Board implementation

-- 32-bit data (seq) locations at address 0x0000
    cs_seq_clock <= BOARD_SEL and CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_0 and CS_0;
  
 -- Configuration data (DAC, ADC, OE, MUX) register module between addresses 0x0100 => 0x01FF    
    cs_cfg <= BOARD_SEL and CS16K_0 and CS4K_0 and CS1K_0 and CS256_1;	  
	CS_CFG_REG <= cs_cfg;

-- Mux register module addresses 0x01FF    
   cs_mux_int <= BOARD_SEL and CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_3 and CS16_3 and CS_F;
 	CS_MUX <= cs_mux_int;
       
-- CS_OE register module addresses 0x01FE    
    CS_OE <= BOARD_SEL and CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_3 and CS16_3 and CS_E;


-- Telemetry (ADC) data register module location between addresses 0x0200 => 0x02FF
    cs_tel <= BOARD_SEL and CS16K_0 and CS4K_0 and CS1K_0 and CS256_2;
    CS_TEL_REG <= cs_tel;
	
-- MicroSequencer Pattern memory selected at 0x1000 to 0x11FF (Actual memory is 2x256x16)
   cs_pat_mem_int <= BOARD_SEL and CS16K_0 and CS4K_1 and CS1K_0 and CS512_0;
	CS_PAT_MEM <= cs_pat_mem_int;
   

--   	cs_micro_control_int <= BOARD_SEL and CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_0 and CS_5;	
--	cs_micro_control_int <= BOARD_SEL and CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_2 and CS_0;

--32-bit data location at address 0x0005
	cs_micro_control_int <= BOARD_SEL and  CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_0 and CS_5;
	CS_MICRO_CONTROL <= cs_micro_control_int;
	
-- Guider CCD registers
	 cs_ccd1 <= BOARD_SEL and CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_0 and CS_1;
	 cs_ccd2 <= BOARD_SEL and CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_0 and CS_2;
	 cs_ccd3 <= BOARD_SEL and CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_0 and CS_3;
	 cs_load <= BOARD_SEL and CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_0 and CS_4; 

------------------------------------------------------------------------
-- CONTROL REGISTER PROCESS
-- Capture the data bus to the control register
------------------------------------------------------------------------
    Cntrl_Reg : Process (SYSCLK)
    begin
        if (rising_edge(SYSCLK)) then
            if (WRITE = '1' and control_cs = '1') then
                    control_reg <= SEQ_DATA_IN(15 downto 0);
                end if;
        end if;
    end process;

------------------------------------------------------------------------
-- CLOCK DATA REGISTER PROCESS
-- Capture the data bus to the clock data register
------------------------------------------------------------------------
    Seq_Reg : process (SYSCLK)
    begin
        if(rising_edge(SYSCLK)) then
            if (WRITE = '1') then
                if (cs_seq_clock = '1') then
                    seq_clock_reg <= SEQ_DATA_IN;
                end if;
            end if;
        end if;
    end process;

------------------------------------------------------------------------
-- STATUS REGISTER
------------------------------------------------------------------------
    status_reg <= "000000" & ANALOG_VOLTAGE_OK & SSN_CRC_OK & CFG_STATUS_IN(7 downto 0);

------------------------------------------------------------------------
-- OUTPUT DATA MUX
------------------------------------------------------------------------
         
    DATA_OUT <= (x"00000000" & control_reg(15 downto 0))   when (control_cs = '1') else (others => 'Z');
    DATA_OUT <= (x"00000000" & status_reg(15 downto 0))    when (status_cs = '1') else (others => 'Z');
    DATA_OUT <= (x"00000000" & board_ident(15 downto 0))   when (board_cs = '1') else (others => 'Z');
    DATA_OUT <= (x"00000000" & code_ident(15 downto 0))    when (code_cs = '1') else (others => 'Z');
--    DATA_OUT <= (x"0000000000" & CFG_FB_DATA_IN(7 downto 0)) when (cs_cfg_fb = '1') else (others => 'Z');
    DATA_OUT <= (x"00000000" & CFG_DATA_IN(15 downto 0))   when (cs_cfg = '1') else (others => 'Z');
    DATA_OUT <= (x"00000000" & TEL_DATA_IN(15 downto 0))   when (cs_tel = '1') else (others => 'Z');
    DATA_OUT <= (x"0000" & seq_clock_reg(31 downto 0))     when (cs_seq_clock ='1') else (others => 'Z');
--    DATA_OUT <= (x"00000000000" & seq_fbias0123_reg(3 downto 0)) when (cs_seq_fbias0123 ='1') else (others => 'Z');
--    DATA_OUT <= (x"00000000000" & seq_fbias4567_reg(3 downto 0)) when (cs_seq_fbias4567 ='1') else (others => 'Z');
    DATA_OUT <= (x"00000000000" & "0" & LED_DATA_IN) when (LedReg_cs = '1') else (others => 'Z');
    DATA_OUT <= (x"0000"  & SSN_DATA_IN(31 downto 0))   when (SNumber_cs = '1') else  (others => 'Z');
    DATA_OUT <= (x"000000000" & "00" & TEMP_DATA_IN(9 downto 0)) when (boardtemp_cs = '1') else  (others => 'Z');
  	DATA_OUT <= (x"00000000" & MICROSEQUENCER_DATA_IN(15 downto 0))  when (cs_pat_mem_int = '1') else (others => 'Z');
  	DATA_OUT <= (x"00000000" & MICRO_CONTROL_IN)  when (cs_micro_control_int = '1') else (others => 'Z');
	DATA_OUT <= (x"000000000" & GUIDER_REG(47 downto 36)) when (cs_load = '1') else  (others => 'Z');
	DATA_OUT <= (x"000000000" & GUIDER_REG(35 downto 24)) when (cs_ccd1 = '1') else  (others => 'Z');
   DATA_OUT <= (x"000000000" & GUIDER_REG(23 downto 12)) when (cs_ccd2 = '1') else  (others => 'Z');
   DATA_OUT <= (x"000000000" & GUIDER_REG(11 downto  0)) when (cs_ccd3 = '1') else  (others => 'Z');
	

------------------------------------------------------------------------
-- IDENTITY CONSTANTS
-- Map the identity module into this code module signals
------------------------------------------------------------------------

    IDENT : Identity_V14 port map (board_ident_out => board_ident, code_ident_out => code_ident);
	
   
end Behavioral;
