------------------------------------------------------------------------
-- PROJECT          :   MONSOON
-- BOARD            :   IR CLOCK&BIAS BOARD
-- DEVICE ID        :   CONFIGURATION CPLD (U119)
-- PARENT MODULE    :   TOP LEVEL
-- MODULE           :   ADC DATA BIT COUNTER
-- VERSION          :   1.25
------------------------------------------------------------------------
-- FILE NAME        :   adc_data_bitcounter.vhd
-- AUTHOR           :   G. Rahmer
-- DATE             :   09 JANUARY 2003
-- COMPANY          :   NOAO
-- PLATFORM         :   Xilinx ISE Webpack 5.1
------------------------------------------------------------------------
-- DESCRIPTION:   
-- Counts twelve (12) SCLK cycles after receiving the ADC_SSTRB signal.
-- During that time, SDO_ENABLE is set for the clock in of the data in
-- the FPGA (going through SDO, external to this module). The SDO_END 
-- signal is asserted at the 12th count to stop the serial clock SCLK 
-- (in module serial_state_ctrl). 
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES:
-- In order to get the proper ADC_SSTRB signal from the selected ADC, all
-- ADC chips must be set to "external clock mode", otherwise the ADC_SSTRB
-- line will not be high-Z when the chip is not selected.
-- To prevent a lock-up situation, a timeout counter is implemented in the 
-- receiving side of the FPGA (cfg_control.vhd)
------------------------------------------------------------------------
-- REVISION HISTORY:
-- DATE       AUTHOR  VERSION    MODIFICATIONS
-- 15/May/03    GR      1.15    Modified the state machine to count 13
--                              sclk's for the start of the SDO reception,
--                              instead of using the SSTRB signal coming
--                              from the ADCs (see corresponding REDFLAG file).
-- June/03      GR      1.2     Changed back to count 12 bits after receiving
--                              the ADC_SSTRB from outside.
--                              Added ADC_SEL to start counting only if in ADC mode
-- 07/July/03   GR      1.25    Not possible to use external ADC_SSTRB. Back to count
--                              SCLK cycles as in 1.15. SDO_ENABLE goes up in the 
--                              falling edge of SCLK.
--
------------------------------------------------------------------------
-- PORT DEFINITIONS:
-- ADC_SEL    : ADC readback in progress
-- SCLK_IN    : Serial data clock input
-- ADC_SSTRB  : Strobe signal from the ADCs to indicate start of
--              serial data.
-- RST        : Incoming reset signal.
-- SDO_END    : Outgoing signal to indicate that 12 bits have been 
--              counted.
-- SDO_ENABLE : Level high during a valid stream of SDO.
--              Used in the FPGA to clock in the data.
------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity adc_data_bitcounter is
    Port (
        sysclk   : in std_logic;
        ADC_SEL : in std_logic;
        SCLK_IN : in std_logic;
        ADC_SSTRB : in std_logic;
        RST : in std_logic;
        SDO_END : out std_logic;
        SDO_ENABLE : out std_logic
    );
end adc_data_bitcounter;

architecture Behavioral of adc_data_bitcounter is

    type statetype is (idle, counting);
    signal state : statetype;
    signal counter : std_logic_vector(4 downto 0);
    
    signal sdo_enbl : std_logic;
    signal sclk_in_reg: std_logic;

begin

--  SDO_ENABLE <= sdo_enbl;

--	process(SCLK_IN)
--	begin
--        if (falling_edge(SCLK_IN)) then
--            SDO_ENABLE <= sdo_enbl;
--        end if;
--    end process;

	sdo_proc: process(sysclk)
	begin
        if rising_edge(sysclk) then
            sclk_in_reg <= sclk_in;
            if (sclk_in='0' and sclk_in_reg='1') then -- falling edge of SCLK_IN
                SDO_ENABLE <= sdo_enbl;
            end if;
        end if;
    end process sdo_proc;

	fsm: process(sysclk)
	begin
        if rising_edge(sysclk) then
            if (sclk_in='1' and sclk_in_reg='0') then -- rising edge of SCLK_IN
                if ( RST = '1' ) then
                    state <= idle;
                    sdo_enbl <= '0';
                    counter <= (others => '0');
                else
                    case state is
                        when idle =>
                            SDO_END <= '0';
                            sdo_enbl <= '0';
                            counter <= (others => '0');
                            if (ADC_SEL = '1') then        -- waiting for ADC_SEL to start counting
                                state <= counting;
                            else
                                state <= idle;
                            end if;
                        when counting =>
                            counter <= counter + '1';
                            if (counter < "10000") then --"01111") then      -- wait during the first 12+4 sclk's
                                sdo_enbl <= '0';
                            elsif (counter < "11100") then --"11011") then   -- enable the SDO reception for 12 more cycles
                                sdo_enbl <= '1';
                            else 
                                SDO_END <= '1';
                                sdo_enbl <= '0';
                                state <= idle;
                            end if;
                    end case;
                end if;
            end if;
        end if;
    end process fsm;

--	process(SCLK_IN)
--	begin
--        if (rising_edge(SCLK_IN)) then
--            if ( RST = '1' ) then
--                state <= idle;
--                sdo_enbl <= '0';
--                counter <= (others => '0');
--            else
--                case state is
--                    when idle =>
--                        SDO_END <= '0';
--                        sdo_enbl <= '0';
--                        counter <= (others => '0');
--                        if (ADC_SEL = '1') then        -- waiting for ADC_SEL to start counting
--                            state <= counting;
--                        else
--                            state <= idle;
--                        end if;
--                    when counting =>
--                        counter <= counter + '1';
--                        if (counter < "10000") then --"01111") then      -- wait during the first 12+4 sclk's
--                            sdo_enbl <= '0';
--                        elsif (counter < "11100") then --"11011") then   -- enable the SDO reception for 12 more cycles
--                            sdo_enbl <= '1';
--                        else 
--                            SDO_END <= '1';
--                            sdo_enbl <= '0';
--                            state <= idle;
--                        end if;
--                end case;
--            end if;
--        end if;
--    end process;

end Behavioral;
