-- Company:    CIEMAT  
-- Engineer:  Juan de Vicente
--
-- Create Date: 10:47:16 11/30/05
-- Design Name: clock board fpga
-- Module Name: cb_fpga_40 - Behavioral
-- Project Name: DES: MonSoon modification to meet DES requeriments
-- Target Device: 
-- Tool versions: 7.1
-- Description: fpga top
-- Port definitions:

-- backplane interface
-- brdclk: clk from the backplane
-- sel_n: selection of this board
-- ack_n: acknoledge to the backplane 
-- seq_data: sequencer data bus. 32-bit writing for sequencing and 16-bit writing for configuration
-- seq_dev_addr: address bus (three-lines) for 32-bit writing
-- seq_mode: indicates the kind of bus transaction: reading(01),16-bit writing (10), 32-bit writing (11)
-- pix_data: 48-bits reading data bus
-- reboot_n: line for restating the fpga program load
-- ssn_dq: 1-Wire bus interfacing with the Silicon Serial Number Device DS2401

-- adc&dac config
-- adc_sstrobe: strobe signals generated by adcs.
-- adc_cs_n: adc chip selection
-- adc_shdn_n: adc shut down signal
-- dac_cs_n: dac chip selection
-- dac_shdn_n: dac shut down signal
-- dac_rst_n: reset for the dac chips
-- sdo: serial data for adc/dac configuration and triggering. Input to the fpga  
-- sdi: Output form the fpga. Serial data configuration and triggering for adc/dac. 
-- sclk: For clocking sdi/sdo serial data

-- sequencer  
--clock A: clocks sequence for CCDs of group A coming from either sequencer or microsequencer 
--clock B: clocks sequence for CCDs of group B coming from either sequencer or microsequencer 
--clock C: clocks sequence for CCDs of group C coming from either sequencer or microsequencer 
--clk_en: signal to connect clocks voltage swing generation paths to the cable drivers 

-- front panel indicator 
-- leds: leds indicators
-- muxa, muxb, muxena, muxenb: front panel clock signal selection. 

-- Board
-- reset_n: reset from the board
-- temp_sclk, temp_dout, temp_din: for reading the board temperature 

-- Test:
-- test_sel, test: signals for prototype testing 

-- Dependencies:
-- Comments: Included a translation table (ROM) Monsoon compatible (clkAaddr=mux(5 downto 0),clkBaddr=mux(11 downto 6))

-- Revision: 04/20/06
-- Revision 0.01 - File Created
-- Additional Comments: integration fpga cpld
-- Revision: 11/16/06- Conversion from mux to realmux for the front panel
-- Revision: 06/16/08- Added IOports drivers control. Added cs_temp.

-- Revision: 11/01/08 
--    Swapped pinout between groups A and C for adapting to CTBv2 (clock transition board version 2)
-- 	New table for conversion mux to realmux according to CTBv2 
--   	Added	analog_voltage_ok port and associated to a bit in the status register 
--
-- Revision: 12/11/08 
-- 	New table for conversion DAC and ADC address according to CTBv2 (cfg_control)
-- 	Added some Jamieson proposals (one clock domain)
--	 	Fixed some test signals for trigger and shutter
--		Code valid for both CBv1 and CBv2
-- Revision: 7/15/09
--    Start microsequencer command pass to be 32 write operation in address 0x05
-- 	Added leds disable in mux_reg(13)
-- Revision: 7/27/09
--     Front panel mux2 mapping error corrected
-- Revision: 11/4/09
--     Included Microsequencer.vhd modified by Jamieson (to sincronize with the acquisition board)
-- Revision v3.4: 18/06/2010
-- 	Included guider_ccd-behavioral module provided by Nick Langellier
-- 	Modifications in cb_fpga.vhd to integrate the guider module (Nick Langellier)
-- Revision v4.00: 03/09/2011
-- 	Updated code version and board version at identity module
--		When reset leds are disable (mux_reg(13)='1')
-- Revision v4.01: 30/03/2011
--	  Board identity programmed to 0x1550
--   Updated serno_interface module (Serial number readout)

--------------------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.ALL;
use ieee.numeric_std.ALL;
use ieee.std_logic_unsigned.ALL;
-- synopsys translate_off
library UNISIM;
use UNISIM.Vcomponents.ALL;
-- synopsys translate_on


entity cb_fpga is
  port (
    --backplane interface
    reset_n  : in  std_logic;  -- from front panel, active high.
    brdclk       : in  std_logic;  -- from backplane 40MHz
    sel_n        : in  std_logic;
    ack_n        : out std_logic;
    seq_data     : in  std_logic_vector (31 downto 0);
    seq_dev_addr : in  std_logic_vector (2 downto 0);
    seq_mode     : in  std_logic_vector (1 downto 0);
    pix_data     : out std_logic_vector (47 downto 0);
	 reboot_n : out   std_logic;
    ssn_dq   :inout std_logic;			 

    --dac/adc config and telemetry 
    adc_sstrobe : in  std_logic_vector (6 downto 0);  --(16 downto 0); 
    adc_cs_n    : out std_logic_vector (6 downto 0);  -- (16 downto 0);  
    adc_shdn_n  : out std_logic;
    dac_cs_n   : out std_logic_vector (5 downto 0);  --(8 downto 0); 
    dac_shdn_n : out std_logic;
    dac_rst_n  : out std_logic;
    sdo : in std_logic;
    sclk_dac : out std_logic;
    sdi_dac  : out std_logic;
    sclk_adc : out std_logic;
    sdi_adc  : out std_logic;

    --clock generation 
    clock_a : out std_logic_vector (11 downto 0);
    clock_b : out std_logic_vector (11 downto 0);
    clock_c : out std_logic_vector (11 downto 0);
    clk_en      : out std_logic;
    led         : out std_logic;
    led_rst_seq : out std_logic;
    led_mux     : out std_logic;
    muxa   : out std_logic_vector (3 downto 0);
    muxb   : out std_logic_vector (3 downto 0);
    muxena : out std_logic_vector (2 downto 0);
    muxenb : out std_logic_vector (2 downto 0);

    --board temperature
    temp_sclk : out std_logic;
    temp_dout : in  std_logic;
    temp_din  : out std_logic;
    test_sel     : in    std_logic_vector (2 downto 0);
    test         : out   std_logic_vector (16 downto 1);

    --added in v2
    cs_temp_ext  : out   std_logic;
    fpga_aux1    : inout std_logic;
    fpga_aux2    : inout std_logic;
    fpga_aux3    : inout std_logic;
    ioports_adir : out   std_logic;
    ioports_bdir : out   std_logic;
    ioports_oe   : out   std_logic;
	 analog_voltage_ok : in std_logic
    );
end cb_fpga;


architecture cb_fpga_arch of cb_fpga is
   attribute BOX_TYPE   								   : STRING ;
   signal adc_data_rdy  								    : std_logic;
   signal addr          								    : std_logic_vector (15 downto 0);
   signal BRD_SEL       								    : std_logic;
   signal cfg_data      								    : std_logic_vector (15 downto 0);
   signal cfg_done      								    : std_logic;
   signal cfg_status    								    : std_logic_vector (7 downto 0);
   signal clock         								    : std_logic;
   signal cs_cfg        								    : std_logic;
   signal cs_tel        								    : std_logic;
   signal DATA          								    : std_logic_vector (31 downto 0);
   signal reset_cfg     								    : std_logic;
   signal RST_ACTIVE    								    : std_logic;
   signal reset 								 			    : std_logic;
   signal tel_data      								    : std_logic_vector (15 downto 0);
   signal write_sd      								    : std_logic;
   signal wr_backplane16								    : std_logic;
   signal wr_backplane32								    : std_logic;
   signal local_data    								    : std_logic_vector (15 downto 0);
   signal data_bp_reg									    : std_logic_vector (7 downto 0);
   signal adc_data      								    : std_logic_vector (15 downto 0);
   signal cs_temp	      								 		: std_logic;
   signal ssn_data      								    : std_logic_vector (47 downto 0);
   signal data_out      								    : std_logic_vector (47 downto 0);
   signal temp_data     								    : std_logic_vector (9 downto 0);
   signal ssn_crc_ok    								    : std_logic;
   signal addr_bp_reg   								    : std_logic_vector (7 downto 0);
   signal reboot_int    								    : std_logic;
   signal led_cs        								    : std_logic;
   signal led_data      								    : std_logic_vector (2 downto 0);
   signal read          								    : std_logic;
   signal write         								    : std_logic;
   signal RST_CMD     									    : std_logic;
   signal high	        								    : std_logic;
   signal low	        								    : std_logic;
   signal CFG_SEL  										    : std_logic_vector (1 downto 0);
   signal CFGCLK	    								    : std_logic;
   signal REBOOT_N_DUMMY								    : std_logic;
   signal sdo_end_int   								    : std_logic;
   signal adc_strobe_int								    : std_logic;
   signal sclk_int	    								    : std_logic;
   signal sdo_enable_int								    : std_logic; 	
   signal adc_sstrb_int 								    : std_logic;	
   signal mux           								    : std_logic_vector (12 downto 0);
   signal RmuxA											    : std_logic_vector (5 downto 0);
   signal RmuxB											    : std_logic_vector (5 downto 0);	 
   signal cs_mux										    : std_logic;
   signal cs_oe											    : std_logic;
   signal cs_pat_mem									    : std_logic;
   signal cs_micro_control_reg								: std_logic;
   signal microseq_clock									: std_logic_vector (30 downto 12);
   signal seq_clock										    : std_logic_vector (31 downto 0);
   signal ENA_MICRO											: std_logic;
   signal oneWireData										: std_logic_vector(7 downto 0); --open
   signal nguard											: std_logic;
   signal h2												: std_logic;
   signal sclk                                              : std_logic; 
   signal sdi											    : std_logic;
   signal CLOCK_DATA  										: std_logic_vector (31 downto 0);
   signal microsequencer_data								: std_logic_vector (15 downto 0);
   signal sequencer_trigger									: std_logic_vector (15 downto 0);
   signal mux_data											: std_logic_vector(15 downto 0);

	signal    guider_data									: std_logic_vector (47 downto 0);
	signal    seq_clock_new									: std_logic_vector (11 downto 0);	

	SIGNAL RESET_N_INT											:std_logic;
    signal address_CTBv2mapping						 	:std_logic_vector(7 downto 0);
	 signal leds_disable									:std_logic;
	 signal led_int        : std_logic;
    signal led_mux_int     : std_logic;
					 
   component seq_control
      port ( sysclk       : in    std_logic; 
             write_32     : in    std_logic; 
             addr         : in    std_logic_vector (7 downto 0); 
             reset        : in    std_logic; 
             data_in      : in    std_logic_vector (31 downto 0); 
             clock_data   : out   std_logic_vector (31 downto 0) 
            ); 
   end component;
   
   component mux_reg 
    port ( SYSCLK 			: in std_logic;
           RESET 			: in std_logic;
           WR_BACKPLANE16 	: in std_logic;
				DATA_BACKPLANE 	: in std_logic_vector(15 downto 0);	
				DATA_OUT_BACKPLANE : out std_logic_vector(15 downto 0);	
				CS_MUX 			: in std_logic;
				MUXA        		: out   std_logic_vector (3 downto 0); 
       	   MUXB        		: out   std_logic_vector (3 downto 0);
				MUXENA        	: out   std_logic_vector (2 downto 0); 
       	   MUXENB        	: out   std_logic_vector (2 downto 0);
				LED_MUX		 	: out std_logic
		);
   	 end component;
    
	component oe_reg 
    port ( SYSCLK 			: in std_logic;
           RESET 			: in std_logic;
           WR_BACKPLANE16 	: in std_logic;
		   DATA_BACKPLANE 	: in std_logic;	
           CS_OE 			: in std_logic;
		   OE					: out std_logic
		   );
    end component;
   
   component cfg_ser_in
      port ( 
				 sysclk       : in std_logic;
				 cfgclk       : in    std_logic; 
             sdata_in     : in    std_logic; 
             sdata_enable : in    std_logic; 
             data_rdy     : out   std_logic; 
             data_out     : out   std_logic_vector (15 downto 0); 
             reset        : in    std_logic);
   end component;
   
      	
   component serial_state_ctrl
    port ( 
			 sysclk  : in std_logic;
		    func_sel : in std_logic_vector(1 downto 0);
		    cfgclk : in std_logic;
          sdo_end : in std_logic;
           SCLK : out std_logic;
           SDI : out std_logic;
		   ADC_STRB: out std_logic;
           RESET : in std_logic;
           WRITE_SD : in std_logic;
           ADDR_BP_REG: in std_logic_vector(7 downto 0);
           DATA_LOCAL_IN : in std_logic_vector(15 downto 0);
           DATA_BP_REG : in std_logic_vector(7 downto 0);
           CFG_DONE : out std_logic;
		   DAC_CS_REG: out std_logic_vector(5 downto 0);
		   ADC_CS_REG: out std_logic_vector(6 downto 0)
           );
	end component;
		
     
   	   
   component cfg_reg_file
      port ( wr_backplane       : in    std_logic; 
             addr_backplane     : in    std_logic_vector (7 downto 0); 
             cs                 : in    std_logic; 
             rst                : in    std_logic; 
             clk                : in    std_logic; 
             addr_local         : in    std_logic_vector (7 downto 0); 
             data_in_backplane  : in    std_logic_vector (15 downto 0); 
             data_out_backplane : out   std_logic_vector (15 downto 0); 
             data_out_local     : out   std_logic_vector (15 downto 0));
   end component;
   component dac_reg_file
      port ( wr_backplane       : in    std_logic; 
             addr_backplane     : in    std_logic_vector (7 downto 0); 
             cs                 : in    std_logic; 
             rst                : in    std_logic; 
             clk                : in    std_logic; 
             data_in_backplane  : in    std_logic_vector (15 downto 0); 
             data_out_backplane : out   std_logic_vector (15 downto 0) 
            );
   end component;

   component tel_data_reg_file
      port ( wr_local           : in    std_logic; 
             addr_backplane     : in    std_logic_vector (7 downto 0); 
             data_in            : in    std_logic_vector (15 downto 0); 
             rst                : in    std_logic; 
             clk                : in    std_logic; 
             data_out_backplane : out   std_logic_vector (15 downto 0); 
             addr_local         : in    std_logic_vector (7 downto 0));
   end component;
   
   component cfg_control
      port ( sysclk           : in    std_logic; 
             wr_backplane     : in    std_logic; 
             adc_data_ready   : in    std_logic; 
             addr_backplane   : in    std_logic_vector (7 downto 0); 
             cfgclk           : out   std_logic; 
             write_sd         : out   std_logic; 
             reset            : in    std_logic; 
             cfg_sel          : out   std_logic_vector (1 downto 0); 
             cfg_status       : out   std_logic_vector (7 downto 0); 
             cs_cfg           : in    std_logic; 
             cs_tel           : in    std_logic;
			 cfg_done         : in    std_logic; 
             data_backplane   : in    std_logic_vector (15 downto 0); 
             addr_bp_reg	  : out   std_logic_vector (7 downto 0); 
             data_bp_reg  	  : out   std_logic_vector (7 downto 0); 
             reset_cfg        : out   std_logic);
   end component;
   
   component BUFT
      port ( I : in    std_logic; 
             T : in    std_logic; 
             O : out   std_logic);
   end component;
   attribute BOX_TYPE of BUFT : COMPONENT is "BLACK_BOX";
   
   component OBUF
      port ( I : in    std_logic; 
             O : out   std_logic);
   end component;
   attribute BOX_TYPE of OBUF : COMPONENT is "BLACK_BOX";
   
     
  
 component TempSensor 
    Port ( SYSCLK    : in  std_logic;
    	     CS_TEMP   : in  std_logic;
		   TEMP_DATA : out std_logic_vector(9 downto 0);
           TEMP_DIN  : out std_logic;
           TEMP_DOUT : in  std_logic;
           TEMP_SCLK : out std_logic;
			  CS_TEMP_EXT : out std_logic; --added in V2
			  RESET : in std_logic --added in V2
         );
end component;
 
   
     
   component cb_personality_v40
      port ( ssn_crc_ok     					  : in    std_logic; 
             tel_data_in    					  : in    std_logic_vector (15 downto 0); 
             cfg_data_in    					  : in    std_logic_vector (15 downto 0);
			 microsequencer_data_in				  :	in    std_logic_vector (15 downto 0);
			 micro_control_in				      : in 	  std_logic_vector (15 downto 0);
			 cfg_status_in  					  : in    std_logic_vector (7 downto 0);
			 ssn_data_in    					  : in    std_logic_vector (31 downto 0); 
             temp_data_in   					  : in    std_logic_vector (9 downto 0); 
             cs_cfg_reg     					  : out   std_logic; 
             cs_tel_reg     					  : out   std_logic; 
			 cs_mux								  : out   std_logic; 
			 cs_oe								  : out   std_logic;
			 cs_temp		   					  : out   std_logic; 
             data_out       					  : out   std_logic_vector (47 downto 0); 
             led_reg_cs     					  : out   std_logic; 
             sysclk         					  : in    std_logic; 
             led_data_in    					  : in    std_logic_vector (2 downto 0); 
             write          					  : in    std_logic; 
             address        					  : in    std_logic_vector (15 downto 0); 
             seq_data_in    					  : in    std_logic_vector (31 downto 0); 
             board_sel      					  : in    std_logic;
			 cs_pat_mem  						  : out 	std_logic;
			 cs_micro_control	    			  : out 	std_logic;
			  analog_voltage_ok				  : in std_logic;
			  guider_reg                     : in  std_logic_vector(47 downto 0)
			  );
   end component;
   
   component ledlatchv40
      port ( sysclk       : in    std_logic; 
             led_cs       : in    std_logic; 
             read         : in    std_logic; 
             reset_active : in    std_logic; 
             led_src_1    : in    std_logic; 
             led_src_2    : in    std_logic; 
             led_src_3    : in    std_logic; 
             data_in      : in    std_logic_vector (2 downto 0); 
             led_en       : out   std_logic; 
             data_out     : out   std_logic_vector (2 downto 0));
   end component;
   
   component seq_ifc_709
      port ( seq_clk   : in    std_logic; 
             brdsel_n  : in    std_logic; 
			 brdack_n  : out   std_logic;
			 			 
             seq_mode  : in    std_logic_vector (1 downto 0); 
             dev_addr  : in    std_logic_vector (2 downto 0); 
             seq_data  : in    std_logic_vector (31 downto 0); 
             data_in   : in    std_logic_vector (47 downto 0); 
             board_sel : out   std_logic; 
              
             reboot_n  : out   std_logic; 
             read      : out   std_logic; 
             write     : out   std_logic; 
             wrt16     : out   std_logic; 
             wrt32     : out   std_logic; 
             pix_data  : out   std_logic_vector (47 downto 0); 
             address   : out   std_logic_vector (15 downto 0); 
             data_out  : out   std_logic_vector (31 downto 0); 
             busy      : in    std_logic; 
             reset     : out   std_logic; 
             sysclk    : out   std_logic);
   end component;
   
   component reset_logic
      port ( sysclk     : in    std_logic; 
             force_reset : in    std_logic; 
				 board_reset: in    std_logic;		
             sync_rcvd  : in    std_logic; 
             async_flag : out   std_logic; 
             reset	  : out   std_logic);
   end component;
   
     	
      
   component adc_data_bitcounter
      port (sysclk   : in std_logic; 
				sclk_in    : in    std_logic; 
             adc_sstrb  : in    std_logic; 
             sdo_end    : out   std_logic; 
             rst        : in    std_logic; 
             sdo_enable : out   std_logic; 
             adc_sel    : in    std_logic);
	end component;
		
   component Microsequencer is
   port ( ADDR_BACKPLANE     		   : in    std_logic_vector (8 downto 0); 
   	    DATA_IN_BACKPLANE  		   : in    std_logic_vector (15 downto 0);
		    DATA_OUT_BACKPLANE  		   : out   std_logic_vector (15 downto 0);  
          CLK                		   : in    std_logic; 
          CS                 		   : in    std_logic;
		  	  SEQUENCER_TRIGGER_OUT		   : out   std_logic_vector(15 downto 0);
		  CS_MICRO_CONTROL_REG		   : in    std_logic;
		  ENA_MICRO					   : out    std_logic; 		  
          RST                		   : in    std_logic; 
          WR_BACKPLANE       		   : in    std_logic; 
			 WR32								: in std_logic;
          CLOCKS			 		   : out   std_logic_vector (30 downto 12)); 
   end component;
	
	component serno_interface
    port (
      clock      : in    std_logic;          -- 40mhz system clock
      reset    : in    std_logic;          
		 --sn_update  : in    std_logic;          -- pulse high to force a read
      sn_data    : out   std_logic_vector(31 downto 0);
      -- sn_busy    : out   std_logic;
      --sn_crc_ok  : out   std_logic;
      sn_io      : inout std_logic
      );
  end component;	
  
  	component DAC_TelAddressConversion is
    port ( ADDR_BACKPLANE : in std_logic_vector(7 downto 0);
           ADDR_BACKPLANE_MOD : out std_logic_vector(7 downto 0);
           CS_CFG : in std_logic;
           CS_TEL : in std_logic);
	end component;
	
	component guider_CCD is
		port (sysclk   : in  std_logic;
				reset    : in  std_logic;
				wrt      : in  std_logic;
				address  : in  std_logic_vector (15 downto 0);
				seq_data : in  std_logic_vector (11 downto 0);
				data_out : out std_logic_vector (47 downto 0);
				din      : in  std_logic_vector (11 downto 0);
				dout     : out std_logic_vector (11 downto 0));
	end component;
	    
begin
   	

   REBOOT_N <= REBOOT_N_DUMMY;  
     
   
   	adc_sstrb_int <= adc_sstrobe(0) or adc_sstrobe(1) or adc_sstrobe(2) or adc_sstrobe(3) or
  						adc_sstrobe(4) or adc_sstrobe(5) or adc_sstrobe(6);
  
   
    mux_reg_inst: mux_reg 
    port map ( SYSCLK =>clock,
					wr_backplane16 => wr_backplane16,
					reset=>reset,
					data_backplane(15 downto 0)=>DATA(15 downto 0),
					data_out_backplane =>mux_data,
					cs_mux=>cs_mux,
					muxa =>MUXA,
					muxb =>MUXB,
					muxena => MUXENA,
					muxenb =>MUXENB,
					led_mux =>LED_MUX_int);
		
	
	oe_reg_inst: oe_reg 
    port map ( sysclk =>clock,
					wr_backplane16 => wr_backplane16,
					reset=>reset,
					data_backplane=>DATA(3),
					cs_oe=>cs_oe,oe => clk_en);
	
   
    seq_control_inst: seq_control
      port map (addr(7 downto 0)=> ADDR(7 downto 0),
					data_in(31 downto 0)=>DATA(31 downto 0),
					reset=>reset,
					sysclk=>clock,write_32=>wr_backplane32,
					clock_data(31 downto 0)=> seq_clock(31 downto 0));

	  guider_CCD_inst: guider_CCD
		port map (sysclk                => clock,
					 reset                 => reset,
					 wrt                   => write,
					 address (15 downto 0) => addr,
					 seq_data(11 downto 0) => data(11 downto 0),
					 data_out(47 downto 0) => guider_data,
					 din     (11 downto 0) => seq_clock(11 downto 0),
					 dout    (11 downto 0) => seq_clock_new); 
			
	  cfg_ser_in_inst : cfg_ser_in
      port map (sysclk => clock ,
					cfgclk=>CFGCLK,
					reset=>reset_cfg,
					sdata_enable=>sdo_enable_int,
					sdata_in=>SDO,
					data_out(15 downto 0)=>adc_data(15 downto 0),
					data_rdy=>adc_data_rdy);
   
		serial_state_ctrl_inst : serial_state_ctrl
			port map( sysclk => clock,
						addr_bp_reg(7 downto 0)=>addr_bp_reg(7 downto 0),
						cfgclk=>CFGCLK,
						func_sel(1 downto 0)=>CFG_SEL(1 downto 0),
						data_bp_reg(7 downto 0)=>data_bp_reg(7 downto 0),
						data_local_in(15 downto 0)=>local_data(15 downto 0),
						ADC_STRB=>adc_strobe_int,
						reset=>reset_cfg,
						write_sd=>write_sd,
						cfg_done=>cfg_done,
						sdo_end => sdo_end_int,
						sclk =>	sclk_int,
						sdi => sdi,
						DAC_CS_REG=>DAC_CS_N,
						ADC_CS_REG=>ADC_CS_N);
  
	   
    adc_data_bitcounter_inst : adc_data_bitcounter
      port map (
					 sysclk  =>clock,
					 adc_sel=>adc_strobe_int,
                adc_sstrb=>adc_sstrb_int,
                rst=> reset,
                sclk_in=>sclk_int,
                sdo_enable=>sdo_enable_int,
                sdo_end=>sdo_end_int);

    DAC_TelAddressConversion_inst: DAC_TelAddressConversion
    port map( 
	 			addr_backplane => ADDR(7 downto 0),
         	ADDR_BACKPLANE_MOD => address_CTBv2mapping,
         	CS_CFG => cs_cfg,
         	CS_TEL => cs_tel);   
  
   cfg_reg_file_inst : cfg_reg_file
      port map (addr_backplane(7 downto 0)=>address_CTBv2mapping, --ADDR(7 downto 0),
					addr_local(7 downto 0)=>addr_bp_reg(7 downto 0),
					clk=>clock, cs=>cs_cfg,
					data_in_backplane(15 downto 0)=>DATA(15 downto 0),
					rst=>reset,
					wr_backplane=>wr_backplane16,
					data_out_backplane(15 downto 0)=>cfg_data(15 downto 0),
					data_out_local(15 downto 0)=>local_data(15 downto 0));
			
	
   
    tel_data_reg_file_inst : tel_data_reg_file
      port map ( addr_local(7 downto 0)=>addr_bp_reg(7 downto 0),
					addr_backplane(7 downto 0)=>address_CTBv2mapping, --ADDR(7 downto 0),
					clk=>clock, data_in(15 downto 0)=>adc_data(15 downto 0),
					rst=>reset,
					wr_local=>adc_data_rdy,
					data_out_backplane(15 downto 0)=>tel_data(15 downto 0));
   
   cfg_control_inst : cfg_control
      port map (adc_data_ready=>adc_data_rdy,
					addr_backplane(7 downto 0)=>address_CTBv2mapping, --ADDR(7 downto 0),
					cfg_done=>cfg_done, cs_cfg=>cs_cfg,
					cs_tel=>cs_tel,
					data_backplane(15 downto 0)=>DATA(15 downto 0),
					reset=>reset,
					sysclk=>clock,
					wr_backplane=>wr_backplane16,
					addr_bp_reg(7 downto 0)=>addr_bp_reg(7 downto 0),
					cfgclk=>CFGCLK,
					cfg_sel(1 downto 0)=>CFG_SEL(1 downto 0),
					cfg_status(7 downto 0)=>cfg_status(7 downto 0),
					data_bp_reg(7 downto 0)=>data_bp_reg(7 downto 0),
					reset_cfg=>reset_cfg, write_sd=>write_sd);
   
   
   tempsensor_inst : tempsensor
      port map (CS_TEMP=>cs_temp,
					temp_dout=>temp_dout,
					sysclk => clock,
					temp_din=>temp_din,
					temp_sclk=>temp_sclk,
					temp_data(9 downto 0)=>temp_data(9 downto 0),
					CS_TEMP_EXT =>CS_TEMP_EXT, RESET => reset);
   
                             
   cb_personality_v40_inst : cb_personality_v40
      port map (address(15 downto 0)=>ADDR(15 downto 0), board_sel=>BRD_SEL,
            cfg_data_in(15 downto 0)=>cfg_data(15 downto 0),cfg_status_in(7 downto 0)=>cfg_status(7 downto 0), led_data_in(2 downto 0)=>led_data(2
            downto 0), seq_data_in(31 downto 0)=>DATA(31 downto 0),
            ssn_crc_ok=>ssn_crc_ok, ssn_data_in(31 downto 0)=>ssn_data(31 downto
            0), sysclk=>clock, tel_data_in(15 downto 0)=>tel_data(15 downto 0),
            temp_data_in(9 downto 0)=>temp_data(9 downto 0), write=>write,
            cs_cfg_reg=>cs_cfg, cs_tel_reg=>cs_tel,
            data_out(47 downto 0)=>data_out(47 downto 0),
            cs_temp=>cs_temp, led_reg_cs=>led_cs,cs_mux=>cs_mux,cs_oe=>cs_oe,
			cs_pat_mem=>cs_pat_mem, cs_micro_control => cs_micro_control_reg,
			microsequencer_data_in=>microsequencer_data,micro_control_in =>sequencer_trigger,
			analog_voltage_ok => ANALOG_VOLTAGE_OK,
			guider_reg(47 downto 0) => guider_data);
   
   ledlatchv40_inst : ledlatchv40
      port map (data_in(2 downto 0)=>DATA(2 downto 0),
				led_cs=>led_cs,
				led_src_1=>BRD_SEL,
				led_src_2=>high,
				led_src_3=>low,
				read=>read, reset_active=>RST_ACTIVE,
				sysclk=>clock,
				data_out(2 downto 0)=>led_data(2 downto 0),
				led_en=>LED_int);
   
   seq_ifc_709_inst : seq_ifc_709
      port map (brdsel_n=>SEL_N,
					busy=>low,
					data_in(47 downto 0)=>data_out(47 downto 0),
					dev_addr(2 downto 0)=>SEQ_DEV_ADDR(2 downto 0),
					seq_clk=>BRDCLK,
					seq_data(31 downto 0)=>SEQ_DATA(31 downto 0),
					seq_mode(1 downto 0)=>SEQ_MODE(1 downto 0),
					address(15 downto 0)=>ADDR(15 downto 0),
					board_sel=>BRD_SEL,
					brdack_n=>ACK_N,
					data_out(31 downto 0)=>DATA(31 downto 0),
					pix_data(47 downto 0)=>PIX_DATA(47 downto 0),
					read=>read,
					reboot_n=>reboot_int,reset=>RST_CMD,
					sysclk=>clock, write=>write,
					wrt16=>wr_backplane16, wrt32=>wr_backplane32);
			
		Microsequencer_inst: Microsequencer
		port map (ADDR_BACKPLANE => ADDR(8 downto 0), 
					 DATA_IN_BACKPLANE => DATA(15 downto 0),
					 DATA_OUT_BACKPLANE => microsequencer_data,
					 SEQUENCER_TRIGGER_OUT => sequencer_trigger,
					 CLK => clock,
					 CS => cs_pat_mem, 
					 CS_MICRO_CONTROL_REG => cs_micro_control_reg,
					 ENA_MICRO	=> ENA_MICRO,
					 RST => reset,
					 WR_BACKPLANE	=> wr_backplane16,
					 WR32 => wr_backplane32,
					 CLOCKS => microseq_clock(30 downto 12)); 
					 
     reset_logic_inst : reset_logic
      port map (
		force_reset =>RST_CMD, -- backplane reset
		board_reset =>reset_n,   -- front panel reset
		sync_rcvd=>BRD_SEL,
		sysclk=>clock,
      async_flag=>RST_ACTIVE,
		reset=> reset);
   
	   
	  
  XLXI_503 : BUFT
      port map (I=>RESET_N_INT, T=>RESET_N_INT, O=>REBOOT_N_DUMMY);
  	RESET_N_INT <= not RESET_N;


  XLXI_140 : BUFT
      port map (I=>reboot_int, T=>reboot_int, O=>REBOOT_N_DUMMY);  
   	  
 serno_inst: serno_interface
  port map(
    clock      => clock,
    reset    => reset, 
    --sn_update  => '0',
    sn_data    => ssn_data(31 downto 0),
    --sn_busy    => open,
    --sn_crc_ok  => ssn_crc_ok,
    sn_io      => ssn_dq);

	 	   
  
  	high <='1';
	low  <='0';
  	DAC_SHDN_N <='1';
   ADC_SHDN_N <='1';
   DAC_RST_N  <='1';  
  	LED_RST_SEQ <=reset and not leds_disable;
  	SDI_DAC	 <= sdi;
	SDI_ADC  <= sdi;
	SCLK_DAC <= sclk_int;
	SCLK_ADC <=	sclk_int;
  
  
  		
	--CLOCK DATA SIGNALS
		
		--(31) nguard
		--(30 downto 12) Horizontal clocks H2, 3x(SW,RG,H3_U,H3_L,H1_U,H1_L) 
		--(11 downto 0) vertical clocks 3x(TG,v3,v2,v1)
	
	   		
  	
	CLOCK_DATA(31) <= seq_clock(31);

	CLOCK_DATA(30 downto 12) <= 
						 microseq_clock(30 downto 12) when ENA_MICRO='1' else
						 seq_clock(30 downto 12); 
       
 	CLOCK_DATA(11 downto 0) <= seq_clock_new; 

 	
	nguard <=CLOCK_DATA(31);
	h2	   <=CLOCK_DATA(30);

 
	-- (11	  	        	...	    			     0 )
	-- (nguard,H2,SW,RG,H3_U,H3_L,H1_U,H1_L,TG,v3,v2,v1) 
 	CLOCK_A(11 downto 0) <=nguard & h2 & CLOCK_DATA(17 downto 12) & CLOCK_DATA(3 downto 0);
	CLOCK_B(11 downto 0) <=nguard & h2 & CLOCK_DATA(23 downto 18) & CLOCK_DATA(7 downto 4);
	CLOCK_C(11 downto 0) <=nguard & h2 & CLOCK_DATA(29 downto 24) & CLOCK_DATA(11 downto 8);
	
	
	--added in V2
	 IOPORTS_OE <='0';	-- ouput enable
	 IOPORTS_ADIR <='0'; -- aux2, aux3 configured as output
	 IOPORTS_BDIR <='0';  --aux1 configured as output

	 FPGA_AUX1 <= mux_data(15); -- fixed for trigger  
	 FPGA_AUX2 <= mux_data(14); --fixed for shutter
	 FPGA_AUX3 <= '0';	
	
	 leds_disable <=mux_data(13);
	 led_mux <= led_mux_int and not leds_disable;
	 led <= led_int and not leds_disable;	  
	--test
		  
	TEST(6) <= mux_data(15);	   --fixed for trigger
  	TEST(1) <= mux_data(14);	   --fixed for shutter
	TEST(5 downto 2) <= "0000";
	TEST(16 downto 7) <= "0000000000";	  
			
end cb_fpga_arch;


