------------------------------------------------------------------------
-- PROJECT          :   MONSOON
-- BOARD            :   IR CLOCK&BIAS BOARD
-- DEVICE ID        :   VIRTEX FPGA (U116)
-- PARENT MODULE    :   TOP LEVEL
-- MODULE           :   ADC READBACK DATA RECEIVER
-- VERSION          :   1.1 
------------------------------------------------------------------------
-- FILE NAME        :   cfg_ser_in.vhd
-- AUTHOR           :   G. Rahmer
-- DATE             :   14 JANUARY 2003
-- COMPANY          :   NOAO
-- PLATFORM         :   Xilinx ISE Webpack 5.1
------------------------------------------------------------------------
-- DESCRIPTION:   
-- This module receives the ADC serial data and
-- converts it into parallel data for to store it in the tel_data_reg_file
-- module.
-- Incoming data (SDATA_IN) is clocked in during SDATA_ENABLE high.
-- After 12 bits are received (the length of the data stream), the data 
-- is available at DATA_OUT, and DATA_RDY is set for a clock cycle, for 
-- cfg_control to perform the corresponding "write" to tel_data_reg_file.
-- 
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES:
-- Serial data should be clocked in with the falling edge of SDATA_CLK
------------------------------------------------------------------------
-- REVISION HISTORY:
-- 24 Oct 2008: change clock from CFGCLK to SYSCLK with an enable from 
--              CFGCLK.
------------------------------------------------------------------------
-- PORT DEFINITIONS:
--
-- CFGCLK : clock signal for the CFG CPLD, generated by "cfg_control"
-- RESET : reset
-- SDATA_IN : serial data from the CFG CPLD
-- SDATA_ENABLE : Level high during a valid stream of SDATA_IN
-- DATA_OUT(15:0) : output data bus to the cfg_reg_file
-- DATA_RDY : data ready at DATA_OUT. Used to write the data in the 
--            tel_data_reg_file
------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity cfg_ser_in is
    Port ( 
			 sysclk       : in std_logic;
			  CFGCLK : in std_logic;
           RESET : in std_logic;
           SDATA_IN : in std_logic;
           SDATA_ENABLE : in std_logic;
           DATA_OUT : out std_logic_vector(15 downto 0);
           DATA_RDY : out std_logic);
end cfg_ser_in;

architecture Behavioral of cfg_ser_in is

    type statetype is (idle, rx_data);
    signal state : statetype;
    signal par_reg : std_logic_vector(15 downto 0);
	 signal cfgclk_reg: std_logic;
begin

    DATA_OUT <= par_reg;
	process(sysclk)
	begin   
        if rising_edge(sysclk) then

            cfgclk_reg <= cfgclk;

            if (cfgclk='1' and cfgclk_reg='0') then  -- rising edge of cfgclk
                if ( RESET = '1' ) then
                    state <= idle;
                    DATA_RDY <= '0';
                    par_reg <= (others => '0');
                else
                    case state is
                        when idle =>
                            DATA_RDY <= '0';
                            par_reg <= (others => '0');
                            if (SDATA_ENABLE = '1') then
                                par_reg <= par_reg(14 downto 0) & SDATA_IN;	 
    							state <= rx_data;
                            else
                                state <= idle;
                            end if;
                        when rx_data =>
                            if (SDATA_ENABLE = '1') then
                                par_reg <= par_reg(14 downto 0) & SDATA_IN;
                                state <= rx_data;
                            else
                                DATA_RDY <= '1';
                                state <= idle;
                            end if;
                    end case;
                end if;
            end if;
        end if;
    end process;
	
--	process(CFGCLK)
--	begin   
--        if (rising_edge(CFGCLK)) then
--            if ( RESET = '1' ) then
--                state <= idle;
--                DATA_RDY <= '0';
--                par_reg <= (others => '0');
--            else
--                case state is
--                    when idle =>
--                        DATA_RDY <= '0';
--                        par_reg <= (others => '0');
--                       if (SDATA_ENABLE = '1') then
--                            par_reg <= par_reg(14 downto 0) & SDATA_IN;	 
--							state <= rx_data;
--                        else
--                            state <= idle;
--                        end if;
--                    when rx_data =>
--                        if (SDATA_ENABLE = '1') then
--                            par_reg <= par_reg(14 downto 0) & SDATA_IN;
--                            state <= rx_data;
--                        else
--                            DATA_RDY <= '1';
--                            state <= idle;
--                        end if;
--                end case;
--            end if;
--        end if;
--    end process;

end Behavioral;
