-- scounter module
-- JTO 6 May 2010
-- Nicholas Langellier  2 June 2010

-- This module is responsible for passing the CB output through
-- untouched (dout = din) or else skipping or masking off X number of
-- pulses after the counter has been loaded.  Fully sync to the system clock.
-- Counts the number of rising edges on din.  Minimum pulse width is one clock
-- period due to the way that the din line is sampled.

-- when masking off the pulses the output level is determined by hold_reg.
-- the value of hold_reg is determined by din at the time the counter is
-- loaded.

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity scounter is
	port (LOAD      : in  std_logic;
			SYSCLK    : in  std_logic;
			RESET     : in  std_logic;
			SKIP_DATA : in  std_logic_vector(11 downto 0);
			DIN       : in  std_logic;
			DOUT      : out std_logic;
			STATUS    : out std_logic);
end scounter;

architecture scounter_arch of scounter is

	signal skip_reg : std_logic_vector (11 downto 0);
	signal hold_reg : std_logic;
	signal din_reg  : std_logic;

begin

	scounter_proc : process(SYSCLK)
	begin
		if rising_edge(SYSCLK) then
			if (RESET = '1') then
				skip_reg <= x"000";
				hold_reg <= '0';
				din_reg  <= '0';
			else
				din_reg <= DIN;
				if (LOAD = '1') then
					hold_reg <= DIN;
					skip_reg <= SKIP_DATA;
				elsif (hold_reg = '1') then
					if (din_reg = '0' and DIN = '1' and skip_reg /= x"000") then
						skip_reg <= skip_reg - 1;
					end if;
				elsif (hold_reg = '0') then
					if (din_reg = '1' and DIN = '0' and skip_reg /= x"000") then
						skip_reg <= skip_reg - 1;
					end if;
				end if;
			end if;
		end if;
	end process scounter_proc;

	STATUS <= '0' when skip_reg = x"000" else '1';
	DOUT   <= DIN when skip_reg = x"000" else hold_reg;

end scounter_arch;

--------------------------------------------------------------------------------
-- GUIDER CCD CODE
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity guider_ccd is
	port (SYSCLK   : in  std_logic;
			RESET    : in  std_logic;
			WRT      : in  std_logic;
			ADDRESS  : in  std_logic_vector(15 downto 0);
			SEQ_DATA : in  std_logic_vector(11 downto 0);
			DATA_OUT : out std_logic_vector(47 downto 0);
			DIN      : in  std_logic_vector(11 downto 0);
			DOUT     : out std_logic_vector(11 downto 0));
end guider_ccd;

architecture behavioral of guider_ccd is

	component scounter
		port (LOAD      : in  std_logic;
				SYSCLK    : in  std_logic;
				RESET     : in  std_logic;
				SKIP_DATA : in  std_logic_vector(11 downto 0);
				DIN       : in  std_logic;
				DOUT      : out std_logic;
				STATUS    : out std_logic);
	end component;

	signal ccd1_reg : std_logic_vector (11 downto 0);
	signal ccd2_reg : std_logic_vector (11 downto 0);
	signal ccd3_reg : std_logic_vector (11 downto 0);
	signal load_reg : std_logic_vector (11 downto 0);

	signal load : std_logic;

begin

	guider_ccd_write : process(SYSCLK)
	begin
		if rising_edge(SYSCLK) then
			if (RESET = '1') then
				ccd1_reg <= x"000";
				ccd2_reg <= x"000";
				ccd3_reg <= x"000";
				load <='0';
			elsif (WRT = '1') then
				if (ADDRESS = x"0001") then
					ccd1_reg <= SEQ_DATA;
				elsif (ADDRESS = x"0002") then
					ccd2_reg <= SEQ_DATA;
				elsif (ADDRESS = x"0003") then
					ccd3_reg <= SEQ_DATA;
				elsif (ADDRESS = x"0004") then
					load <= '1';
				end if;
			elsif (WRT = '0' and load = '1') then
				load <= '0';
			end if;
		end if;
	end process guider_ccd_write;

	DATA_OUT <= load_reg & ccd1_reg & ccd2_reg & ccd3_reg;

	CCD1_TG : scounter
		port map (load, SYSCLK, RESET, ccd1_reg, DIN(11), DOUT(11), load_reg(11));
	CCD1_V3 : scounter
		port map (load, SYSCLK, RESET, ccd1_reg, DIN(10), DOUT(10), load_reg(10));
	CCD1_V2 : scounter
		port map (load, SYSCLK, RESET, ccd1_reg, DIN(9),  DOUT(9),  load_reg(9));
	CCD1_V1 : scounter
		port map (load, SYSCLK, RESET, ccd1_reg, DIN(8),  DOUT(8),  load_reg(8));
	CCD2_TG : scounter
		port map (load, SYSCLK, RESET, ccd2_reg, DIN(7),  DOUT(7),  load_reg(7));
	CCD2_V3 : scounter
		port map (load, SYSCLK, RESET, ccd2_reg, DIN(6),  DOUT(6),  load_reg(6));
	CCD2_V2 : scounter
		port map (load, SYSCLK, RESET, ccd2_reg, DIN(5),  DOUT(5),  load_reg(5));
	CCD2_V1 : scounter
		port map (load, SYSCLK, RESET, ccd2_reg, DIN(4),  DOUT(4),  load_reg(4));
	CCD3_TG : scounter
		port map (load, SYSCLK, RESET, ccd3_reg, DIN(3),  DOUT(3),  load_reg(3));
	CCD3_V3 : scounter
		port map (load, SYSCLK, RESET, ccd3_reg, DIN(2),  DOUT(2),  load_reg(2));
	CCD3_V2 : scounter
		port map (load, SYSCLK, RESET, ccd3_reg, DIN(1),  DOUT(1),  load_reg(1));
	CCD3_V1 : scounter
		port map (load, SYSCLK, RESET, ccd3_reg, DIN(0),  DOUT(0),  load_reg(0));

end Behavioral;
