------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   IR ACQUISITION BOARD
-- DEVICE ID    :   CLOCK&BIAS FPGA U116
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   RESET LOGIC FOR CLOCK&BIAS FPGA
-- VERSION      :   4.0 
------------------------------------------------------------------------
-- FILE NAME    :   RESET_LOGIC.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   04/29/2004
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 5.2
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- TAKES AS INPUT THE SEQUENCER BUS RESET AND PRODUCES ONE RESET
-- PULSE LOCAL TO THE DEVICE + A FLAG TO INDICATE THAT RESET HAS OCCURED.
-- ALSO PROVIDES THE BOOT UP RESET PULSE.
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- RESET PULSE DURATION IS 15ms ON BOOT ELSE 1us FOR OTHER SOURCES
-- THE NOMINAL BOOT PERIOD IS 200MS WITH A CONFIG RATE OF 34. 
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE	      AUTHOR  VERSION  MODIFICATIONS
-- 04/29/2004   PCM   4.0   ORIGINAL VERSION ADAPTED FROM MASTER CONTROL BOARD SEQ FPGA
-- 30 Oct 2008  JTO         remove timers, add input for board reset, add sync reset, reset_n outputs
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- SYSCLK       : SYSTEM CLOCK SOURCE
-- RST_CMD      : SOFT RESET REQUEST STROBE FROM PAN
-- SYNC_RCVD    : SYNCHRONIZATION WORD RECEIVED FROM PAN
-- ASYNC_FLAG   : AFTER RESET, THIS IS SET TO INDICATE THAT SYNCHRONIZATION WITH PAN MAY HAVE BEEN LOST
-- RESET_OUT    : NOMINAL, LOCAL RESET SIGNAL.
------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity reset_logic is
    port ( 
        sysclk      : in  std_logic;
        force_reset : in  std_logic;  -- from backplane
        board_reset : in  std_logic;  -- from front panel
        sync_rcvd   : in  std_logic;
        async_flag  : out std_logic;
        reset       : out std_logic
        );
end reset_logic;

architecture reset_logic_arch of reset_logic is

    signal async_flag_reg : std_logic;
    signal reset_reg      : std_logic;

begin

    reset_proc: process (sysclk)
    begin
        if rising_edge(sysclk) then
            reset_reg <= force_reset or board_reset;
        end if;
    end process reset_proc;

    reset   <= reset_reg;

    asynflag_proc: process (sysclk)
    begin
        if rising_edge(sysclk) then
            if (reset_reg = '1') then
                async_flag_reg <= '1';
            elsif (sync_rcvd = '1') then
                async_flag_reg <= '0';
            end if;
        end if;
    end process asynflag_proc;

    async_flag <= async_flag_reg;

end reset_logic_arch;
