------------------------------------------------------------------------
-- PROJECT          :   DES (MONSOON modification)
-- BOARD            :   CLOCK BOARD
-- DEVICE ID        :   FPGA
-- PARENT MODULE    :   TOP LEVEL
-- MODULE           :   DAC AND ADC CONFIGURATION
-- VERSION          :   1.0 
------------------------------------------------------------------------
-- FILE NAME        :   serial_state_ctrl.vhd
-- AUTHOR           :   Juan de Vicente (Monsoon G. Rahmer)
-- DATE             :   20 Apr 2006
-- COMPANY          :   CIEMAT
-- PLATFORM         :   Xilinx ISE Webpack 7.1
------------------------------------------------------------------------
-- DESCRIPTION:   
-- This module generates the configuration serial data stream (SDI) and
-- clock (SCLK) for the DACs and ADCs.
-- This stream is assembled out of parallel  data provided by either the backplane (DAC) or the 
-- cfg_reg_file (ADC), and of address information coming from the backplane.
-- There are two different types of streams to be generated, according
-- to FUNC_SEL:

--          00->DAC
--          01->ADC
--          10->OE  Not used
--          11->MUX	Not used

--
-- In the case of the DAC/ADC:
--	 data DAC_CHIP(5) ADC_CHIP(5) are the ADDR vector, 
--The different stream formats (12 bits) are as follows:
--      DAC:
--          | DAC_CHAN(4) |   DAC_Data(8)  |
--      ADC:
--          | fill(4) |  Control_Byte (8)  |

--    "Fill" bits are 0's for ADC.
--
--    Control_Byte format is | 1(Start) | ADC_CHAN(3) | ADC_Data(4) |
--    For the details of the ADC_Data, see MAX1270 datasheet.

-- DAC_CHIP and DAC_CHAN fields are taken from the backplane address bus.
-- DAC_Data is taken from the backplane data bus.
--
-- ADC_CHIP and ADC_CHAN are taken from the backplane address bus.
-- ADC_Data is taken from the local data bus (coming from "cfg_reg_file").

--
-- Data stream generation is triggered by WRITE_SD.
--
-- When in ADC mode, after the "Control byte" data has been sent out,
-- the serial clock SCLK stays on until the data through SDO (outside
-- this module) has been read. This "end of frame" is indicated by the
-- SDO_END signal, coming from the adc_data_bitcounter module.
--

------------------------------------------------------------------------
-- IMPLEMENTATION NOTES:
-- The logic of this module is clocked on the falling edge of CFGCLK to
-- ensure proper timing.

------------------------------------------------------------------------
-- IMPLEMENTATION NOTES:
--
-- The DAC chips have 12 channels each, and the DAC memory area starts at h00,
-- therefore:
--     
--     
--      dac_chip = ADDR_BP_REG_IN(7:4)    before dac_chip = ADDR_BP_REG_IN(7:0) DIV 12 
--      dac_chan = ADDR_BP_REG_IN(3:0)    before dac_chan = ADDR_BP_REG_IN(7:0) MOD 12

-- 
-- The ADC chips have 8 channels each, and the ADC memory area starts at h70,
-- therefore:
--      adc_chip = ADDR_BP_REG_IN(7:3) - "01110"
--      adc_chan = ADDR_BP_REG_IN(2:0)
--      
------------------------------------------------------------------------



------------------------------------------------------------------------
-- REVISION HISTORY:
-- DATE       AUTHOR  VERSION   MODIFICATIONS
-- 09/Jul/03    GR      1.11    DAC_RST is set to "1" after the first write to DAC
--                              Power-up default should be zero.
--                              (not implemented)
-- 04/Dec/03    GR      1.3     Changed the definition for the MUX serial stream
--                              (no real changes here, but in the top-level schematic) 
-- 20/Apr/06    JU		1.0     Modifications for DES
-- 24 Oct 08    JO              sync to sysclk
------------------------------------------------------------------------
-- PORT DEFINITIONS:
--

-- FUNC_SEL(1:0)   : input that selects the function of the incoming data.
-- SDO_END         : input from the adc_data_bitcounter module to indicate
--                   the end of a DAC data frame received.
-- RST             : reset
-- SCLK            : serial clock output to the DAC/ADC.
-- SDI             : serial data output to the DAC/ADC.

-- CFGCLK : clock signal  generated by "cfg_control"
-- RESET : no utilizada
-- WRITE_SD : start of serial stream generation
-- ADDR_BP_REG(7:0) : backplane address latched in cfg_control
-- DATA_LOCAL_IN(15:0) : local data bus from cfg_reg_file
-- DATA_BP_REG(7:0) : backplane data latched in cfg_control
-- CFG_DONE : goes high at the end of the tx of the stream. Tells the 
--            cfg_control to stop CFGCLK (except for the ADCs)
------------------------------------------------------------------------


-- DESCRIPTION:   
-- This module provides the serial data stream for the configuration of DACs and ADCs
  
 

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--use IEEE.Numeric_std.ALL

entity serial_state_ctrl is
    Port ( 
           sysclk : in std_logic;  -- master clock
		   CFGCLK : in std_logic;  -- now an enable pulse

		   FUNC_SEL : in std_logic_vector(1 downto 0);
           SDO_END : in std_logic;
           -- RST : in std_logic;
           SCLK : out std_logic;
           SDI : out std_logic;
           ADC_STRB :out std_logic;
		   RESET : in std_logic;
           WRITE_SD : in std_logic;
           ADDR_BP_REG : in std_logic_vector(7 downto 0);
           DATA_LOCAL_IN : in std_logic_vector(15 downto 0);
           DATA_BP_REG : in std_logic_vector(7 downto 0);
           CFG_DONE : out std_logic;
		   DAC_CS_REG: out std_logic_vector(5 downto 0);
		   ADC_CS_REG: out std_logic_vector(6 downto 0)
           );
end serial_state_ctrl;

architecture Behavioral of serial_state_ctrl is

    signal adc_cfg : std_logic;
    signal dac_cfg : std_logic;    
    type   dac_statetype is (dac_idle, dac_send_data);
    signal dac_state : dac_statetype;
    signal dac_counter : std_logic_vector(3 downto 0);
    signal dac_clken : std_logic;
    signal dac_sdi : std_logic;

    type adc_statetype is (adc_idle, adc_send_data, adc_receive_data);
    signal adc_state : adc_statetype;
    signal adc_counter : std_logic_vector(3 downto 0);
    signal adc_clken : std_logic;
    signal adc_sdi : std_logic;    

	signal dac_cs   : std_logic_vector(5 downto 0);
	signal dac_chip : std_logic_vector(3 downto 0);
    signal dac_chan : std_logic_vector(3 downto 0);		
	signal dac_data : std_logic_vector(7 downto 0);	    
	signal dac_OpReg : std_logic_vector(11 downto 0);
   	
	signal adc_cs   : std_logic_vector(6 downto 0);		
	signal adc_chip : std_logic_vector(4 downto 0);	    
   	signal adc_chan : std_logic_vector(2 downto 0);		
   	signal adc_data : std_logic_vector(3 downto 0);		
	signal adc_OpReg : std_logic_vector(11 downto 0);
		
	signal dac_cfg_done:std_logic;
    signal adc_cfg_done:std_logic;
	
    signal cfgclk_reg : std_logic;
    signal sclk_reg : std_logic;
   			
begin
		
    dac_data <= DATA_BP_REG(7 downto 0);
	dac_chip <= ADDR_BP_REG(7 downto 4);
    dac_chan <= ADDR_BP_REG(3 downto 0);

    --dac_OpReg <= dac_chan(3 downto 0) & dac_data;

    adc_chip <= (ADDR_BP_REG(7 downto 3) - "01110");  
    adc_chan <= ADDR_BP_REG(2 downto 0);
    adc_data <=  DATA_LOCAL_IN(3 downto 0);
   -- adc_OpReg <= "0000" & "1" & adc_chan & adc_data;



	dac_cs <= "000001" when (DAC_CHIP = "0000") else
	          "000010" when (DAC_CHIP = "0001") else
	          "000100" when (DAC_CHIP = "0010") else
              "001000" when (DAC_CHIP = "0011") else
              "010000" when (DAC_CHIP = "0100") else
              "100000" when (DAC_CHIP = "0101") else
			  "000000";
			   
	adc_cs <= "0000001" when (ADC_CHIP = "00000") else
	          "0000010" when (ADC_CHIP = "00001") else
	          "0000100" when (ADC_CHIP = "00010") else
              "0001000" when (ADC_CHIP = "00011") else
              "0010000" when (ADC_CHIP = "00100") else
              "0100000" when (ADC_CHIP = "00101") else
              "1000000" when (ADC_CHIP = "00110") else
              "0000000"; 			  
			                
          
 
   	dac_cfg <= '1' when (FUNC_SEL = "00") else '0';
    adc_cfg <= '1' when (FUNC_SEL = "01") else '0';
     
	--serial output	

	-- SCLK <= ((dac_clken or adc_clken) and CFGCLK);

    sclk_proc: process(sysclk)  -- square this up with a flip flop
    begin
        if rising_edge(sysclk) then
            sclk_reg <= (dac_clken or adc_clken) and CFGCLK;
        end if;
    end process sclk_proc;

    SCLK <= sclk_reg;


    SDI <= dac_sdi when (dac_cfg = '1') else
           adc_sdi when (adc_cfg = '1') else
           '0';
   
	CFG_DONE <= dac_cfg_done or adc_cfg_done;

-- Don't use gated clock CFGCLK to clock logic directly.
-- use SYSCLK to clock logic but use CFGCLK as an enable

dac_configuration_fsm: process(sysclk)
begin
    if rising_edge(sysclk) then
        cfgclk_reg <= cfgclk;
        if (cfgclk='0' and cfgclk_reg='1') then -- FALLING edge of CFGCLK
            if ( reset = '1' or dac_cfg = '0' ) then
                dac_state <= dac_idle;
                dac_clken <= '0';
                dac_sdi <= '0';
    			DAC_CS_REG <= (others => '1');
    			dac_counter <= (others => '0');
    			dac_OpReg <= (others => '0');
    			dac_cfg_done <='0'; --CFG_DONE <= '0';
            else
                case dac_state is 
                    when dac_idle =>                -- Idle state for DAC configuration
    				    dac_clken <= '0';
    					dac_sdi <= '0';
    				   	dac_counter <= (others => '0');
    					dac_cfg_done <='0';  --CFG_DONE <= '0'; 
       				   	DAC_CS_REG <= (others => '1');
    				   
                        if (WRITE_SD = '1' and dac_cfg ='1') then
                       		dac_OpReg <= dac_chan(3 downto 0) & dac_data;
                        	dac_state <= dac_send_data;
    					else
    				   	    dac_state <= dac_idle;
    		  			end if;
                    when dac_send_data =>
    						dac_sdi <= dac_OpReg(11);  --send out the configuration bits
    					   if (dac_counter < "1011") then
    
    						   	dac_clken <= '1';
    							DAC_CS_REG <= not(dac_cs);
    							dac_OpReg <= dac_OpReg(10 downto 0) & '1';
    							dac_counter <= dac_counter + '1';
    					     --	dac_state <= dac_send_data;
    			  		else 
    					 	dac_cfg_done <='1'; --CFG_DONE <= '1';
    			     		dac_state <= dac_idle;
     			 		end if;
				end case;
            end if;
		end if;
    end if;
end process dac_configuration_fsm;



adc_configuration_fsm: process(sysclk)
begin
    if rising_edge(sysclk) then
        if (cfgclk='0' and cfgclk_reg='1') then -- this is the FALLING edge of CFGCLK
            if ( reset = '1' or adc_cfg = '0' ) then
                adc_state <= adc_idle;
                ADC_CS_REG <= (others => '1');  --ADC_STRB <= '0';
                adc_clken <= '0';
                adc_sdi <= '0';
    			adc_counter <= (others => '0');
    			adc_OpReg <= (others => '0');
    			ADC_STRB <= '0';
    			adc_cfg_done <='0'; 
    		 else
                case adc_state is 
                    when adc_idle =>                -- Idle state for ADC configuration
                         adc_OpReg <= "0000" & "1" & adc_chan & adc_data;
    					 adc_cfg_done <='0'; 
                        if (WRITE_SD = '1'and adc_cfg = '1') then
    					    adc_state <= adc_send_data; 
    					else
    						adc_clken <= '0';
                        	adc_sdi <= '0';
                        	ADC_CS_REG <= (others => '1'); --ADC_STRB <= '0';
    						adc_state <= adc_idle;
    						adc_counter <= (others => '0');
    						ADC_STRB <= '0';
    					end if;
                   	when adc_send_data =>
		     						
    					if (adc_counter < "1101") then  -- 12 bits (4 fill, 8 Control_Byte) 
    						adc_clken <= '1';
    						ADC_CS_REG <= not(adc_cs);   --ADC_STRB <= '1';
    						adc_sdi <= adc_OpReg(11);  -- Send out the ADC config data
    				   		adc_OpReg <= adc_OpReg(10 downto 0) & '1';
    						ADC_STRB <= '1';   	       			                  
                  		    adc_counter <= adc_counter + '1';
    				  		adc_state <= adc_send_data;
    			  		else 
    			     		adc_state <= adc_receive_data;
     			 		end if;
    				when adc_receive_data =>
    	        		if (SDO_END = '0') then 
         	       			adc_clken <= '1';   -- Keep the sclk enabled until the adc data has been received
                            ADC_CS_REG <= not(adc_cs);  --ADC_STRB <= '1';
    			     		adc_sdi <= '0';
    				  		adc_state <= adc_receive_data;
    						ADC_STRB <= '1';  
    			  		else 
    						adc_cfg_done <='1';
    			     		adc_state <= adc_idle;
                            ADC_CS_REG <= (others => '1');   --ADC_STRB <= '0';
     			 		end if;
        			end case;
                end if;
		end if;
    end if;
end process adc_configuration_fsm;

--dac_configuration_fsm: process(CFGCLK)
--begin
--    if (falling_edge(CFGCLK)) then
--        if ( reset = '1' or dac_cfg = '0' ) then
--            dac_state <= dac_idle;
--            dac_clken <= '0';
--            dac_sdi <= '0';
--			DAC_CS_REG <= (others => '1');
--			dac_counter <= (others => '0');
--			dac_OpReg <= (others => '0');
--			dac_cfg_done <='0'; --CFG_DONE <= '0';
--        else
--            case dac_state is 
--                when dac_idle =>                -- Idle state for DAC configuration
--				    dac_clken <= '0';
--					dac_sdi <= '0';
--				   	dac_counter <= (others => '0');
--					dac_cfg_done <='0';  --CFG_DONE <= '0'; 
--   				   	DAC_CS_REG <= (others => '1');
--				   
--                    if (WRITE_SD = '1' and dac_cfg ='1') then
--                   		dac_OpReg <= dac_chan(3 downto 0) & dac_data;
--                    	dac_state <= dac_send_data;
--					else
--				   	    dac_state <= dac_idle;
--		  			end if;
--                when dac_send_data =>
--						dac_sdi <= dac_OpReg(11);  --send out the configuration bits
--					   if (dac_counter < "1011") then
--
--						   	dac_clken <= '1';
--							DAC_CS_REG <= not(dac_cs);
--							dac_OpReg <= dac_OpReg(10 downto 0) & '1';
--							dac_counter <= dac_counter + '1';
--					     --	dac_state <= dac_send_data;
--			  		else 
--					 	dac_cfg_done <='1'; --CFG_DONE <= '1';
--			     		dac_state <= dac_idle;
-- 			 		end if;
--					
--			end case;
--		end if;
--    end if;
--end process;
--
--
--
--
--
--adc_configuration_fsm: process(CFGCLK)
--begin
--    if (falling_edge(CFGCLK)) then
--        if ( reset = '1' or adc_cfg = '0' ) then
--            adc_state <= adc_idle;
--            ADC_CS_REG <= (others => '1');  --ADC_STRB <= '0';
--            adc_clken <= '0';
--            adc_sdi <= '0';
--			adc_counter <= (others => '0');
--			adc_OpReg <= (others => '0');
--			ADC_STRB <= '0';
--			adc_cfg_done <='0'; 
--		 else
--            case adc_state is 
--                when adc_idle =>                -- Idle state for ADC configuration
--                     adc_OpReg <= "0000" & "1" & adc_chan & adc_data;
--					 adc_cfg_done <='0'; 
--                    if (WRITE_SD = '1'and adc_cfg = '1') then
--					    adc_state <= adc_send_data; 
--					else
--						adc_clken <= '0';
--                    	adc_sdi <= '0';
--                    	ADC_CS_REG <= (others => '1'); --ADC_STRB <= '0';
--						adc_state <= adc_idle;
--						adc_counter <= (others => '0');
--						ADC_STRB <= '0';
--					end if;
--               	when adc_send_data =>
--		     						
--					if (adc_counter < "1101") then  -- 12 bits (4 fill, 8 Control_Byte) 
--						adc_clken <= '1';
--						ADC_CS_REG <= not(adc_cs);   --ADC_STRB <= '1';
--						adc_sdi <= adc_OpReg(11);  -- Send out the ADC config data
--				   		adc_OpReg <= adc_OpReg(10 downto 0) & '1';
--						ADC_STRB <= '1';   	       			                  
--              		    adc_counter <= adc_counter + '1';
--				  		adc_state <= adc_send_data;
--			  		else 
--			     		adc_state <= adc_receive_data;
-- 			 		end if;
--				when adc_receive_data =>
--	        		if (SDO_END = '0') then 
--     	       			adc_clken <= '1';   -- Keep the sclk enabled until the adc data has been received
--                        ADC_CS_REG <= not(adc_cs);  --ADC_STRB <= '1';
--			     		adc_sdi <= '0';
--				  		adc_state <= adc_receive_data;
--						ADC_STRB <= '1';  
--			  		else 
--						adc_cfg_done <='1';
--			     		adc_state <= adc_idle;
--                        ADC_CS_REG <= (others => '1');   --ADC_STRB <= '0';
-- 			 		end if;
--			end case;
--		end if;
--    end if;
--end process;

end Behavioral;























